@@ -20,24 +20,24 @@
 /*     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.         */
 /* --------------------------------------------------------------------  */
 
-/* ALL function prototypes for the program "CalculiX GraphiX (cgx)" */
+#define     VERSION         "2.23"
+
+/* function prototypes for the program "CalculiX GraphiX (cgx)" */
 
 #include "extUtil.h"
 #include "trackball.h"
 
-/* 
-choose the compiler option "-D SGI_MIPS" in the Makefile for best results 
-if you are working on a sgi workstation or
-if you are working on a linux system with 8-Bit color depth. In this case 
-rendering is done in the index-mode.
-*/
-#ifdef SGI_MIPS
-  #define     INX_MODE    1
-  #define     TEX_MODE    0
+/* change from sem_open to sem_init */
+#ifdef SEMINIT
 #else
-  #define     INX_MODE    0
-  #define     TEX_MODE    1
-#endif
+#define sem_n *nptr
+#define sem_g *gptr
+#define sem_rep *rptr
+#define sem_stn *sptr
+#define sem_nurs *nuptr
+#define sem_map3d *mptr
+#define sem_cute *cptr
+#endif    
 
 
 /* factors for the geometric tolerance for merging */
@@ -53,11 +53,17 @@ rendering is done in the index-mode.
 #define     GTOL_COS_A 0.9914448
 /* defines the minimum angle difference before the line bias is increased */
 #define     MIN_DCOS_A 0.004 
+/* defines the default minimum-element-length/maximum-element-length, used at curves */
+#define     ELEM_LENGTH_RATIO 0.3
+/* normal vector angle difference used to collect a target surface made of faces for 'qdis' qaddTol=QADDTOL */
+#define     QADDTOL   10.
 /* Defines the default maximum distance between nodes during automatic line division calculation */
 /* GTOL_NODE_DIST * gtol = maximum distance between nodes */
 #define     GTOL_NODE_DIST 5.e2
-/* defines the default minimum-element-length/maximum-element-length, used at curves */
-#define     ELEM_LENGTH_RATIO 0.3
+/* amplification factor for the automatic mapping in the graph() function */
+#define     GTOL_FACTOR_GRAPH 1e-4
+/* amplification factor for general mapping */
+#define     GTOL_FACTOR_MAP 1e-2
 
 /* meshing parameters for the unstructured tr3 mesher (mesh2d) */
 #define     NADAPT   8                 // for mesh2d()
@@ -70,27 +76,36 @@ rendering is done in the index-mode.
 #define     ALPHAFACTOR 0.7            // modifies alpha in case of bad elems (in both directions)
 #define     MAX_MESH2D_LOOPS 3              // max refinement loops of mesh2d()
 #define     MESH2D_QUALITY_THRESHOLD 0.01     // qfactor=dist_cg_nurbs/circumlength_of_tri (curvature, 1st parameter)
-
+#define     MAX_REFINEMENT_LOOPS     0     // nr of loops to adjust line divisions on element edges
 /* element splitter parameter to force a certain maximum distance of the element cg to the nurbs */
 #define     TRISPLITTER_QUALITY_THRESHOLD_FACTOR 0.3 // dist_nurbs/v_n1n2 *TRISPLITTER_QUALITY_THRESHOLD_FACTOR, to modify the req. curv in triSplitter()
 
+/* default nr of cpu's to be used */
+#define     NTHREADS_MIN       1
+#define     NTHREADS_MAX       8
 
 /* default location of the help files, please insert the actual one */
 #ifdef MTU
   #ifdef DEVEL
-    #define     HELPFILE        {"/yaprod/yae55/Linux_x86_64/bin/cgx_doc/cgx.html",\
-                             "/yaprod/yae55/Linux_x86_64/bin/ccx_doc/ccx.html"}
+    #define     HELPFILE        {"/yaprod/struct/app/cgx/prod/doc/cgx.html",\
+                             "/yaprod/struct/app/ccx/prod/doc/ccx.html"}
   #else
-    #define     HELPFILE        {"/yaprod/cae/application/CalculiX/doc/cgx_2.13/cgx.html",\
-                             "/yaprod/cae/application/CalculiX/doc/ccx_2.13/ccx.html"}
+    #define     HELPFILE        {"/yaprod/cae/application/CalculiX/doc/cgx_2.23/cgx.html",\
+                             "/yaprod/cae/application/CalculiX/doc/ccx_2.23/ccx.html"}
   #endif
+  /* default tet mesher, 0:Netgen ng_vol, 1:tetgen */
+  #define     TETMESHER  0
 #elif defined AFLIB
-  #define     HELPFILE        {"/usr/local/CalculiX/cgx_2.13/doc/cgx/cgx.html",\
-    "/usr/local/CalculiX/ccx_2.13/doc/ccx/ccx.html",\
-    "/usr/local/CalculiX/cgx_2.13/doc/aflib/aflib.pdf"}
+  #define     HELPFILE        {"/usr/local/CalculiX/cgx_2.23/doc/cgx/cgx.html",\
+    "/usr/local/CalculiX/ccx_2.23/doc/ccx/ccx.html",\
+    "/usr/local/CalculiX/cgx_2.16/doc/aflib/aflib.pdf"}
+  /* default tet mesher, 0:Netgen ng_vol, 1:tetgen */
+  #define     TETMESHER  1
 #else
-  #define     HELPFILE        {"/usr/local/CalculiX/cgx_2.13/doc/cgx/cgx.html",\
-    "/usr/local/CalculiX/ccx_2.13/doc/ccx/ccx.html"}
+  #define     HELPFILE        {"/usr/local/CalculiX/cgx_2.23/doc/cgx/cgx.html",\
+    "/usr/local/CalculiX/ccx_2.23/doc/ccx/ccx.html"}
+  /* default tet mesher, 0:Netgen ng_vol, 1:tetgen */
+  #define     TETMESHER  1
 #endif
 
 #define     INITFILE        {".cgx"}
@@ -100,24 +115,32 @@ rendering is done in the index-mode.
 #ifdef MTU
   #define     BROWSER         {"firefox"}
   #define     PSVIEWER         {"gv"}
+  #define     VIEWFORMAT         {"ps"}
+  #define     ALLOW_SYS_FLAG   1
 #else
-/* #define     BROWSER         {"netscape"} */
-/*   #define     BROWSER         {"mozilla"} */
   #define     BROWSER         {"firefox"}
-/* #define     PSVIEWER         {"ghostview"} */
-  #define     PSVIEWER         {"gv"}
+  #ifdef WIN32
+    #define     PSVIEWER         {"mspaint.exe"}
+    #define     VIEWFORMAT       {"png"}
+  #else
+    #define     PSVIEWER         {"gv"}
+    #define     VIEWFORMAT       {"ps"}
+  #endif
+  #define     ALLOW_SYS_FLAG   0
 #endif
 
 /* more parameters, better do not change them! */
+#define     MIN_ELEM_EDGE_DISTANCE 6e-5  /* elem edges vanish during zooming if smaller */
 #define     TEX_PIXELS  512          /* number of possible colors in texture mode */
 #define     GAMMA       1.9          /* gamma lightens or darkens an image (Hcpy) */
+#define     BLEND_ALPHA 0.33         /* default alpha used for blending */ 
 #define     INI_SCREEN      600
 #define     INI_MENU_WIDTH  184
 #define     INI_MENU_HEIGHT  72
 #define     PS_DENSITY       107     /* basic-density to convert xwd to ps */
 #define     Z_DEPTH          2.      /* edge length of the drawing cube */
-#define     SET_COLS    9            /* no. of predefined colors */
-#define     BAS_COLS    3            /* basic colours: black, white, nickel */
+#define     SET_COLS    11           /* no. of predefined colors */
+#define     BAS_COLS    3            /* basic colours: black, white, neutral */
 #define     DEF_COL     2            /* default colour */
 #define     PICK        1            /* lower nurbs-density during picking */
 #define     PICK_LENGTH 5            /* default cursor-length during picking */
@@ -126,7 +149,8 @@ rendering is done in the index-mode.
 #define     MIN_LINE_DIV 2           /* minimum line division used for the automatic division calculation */
 #define     NURS_TOLERANCE 30
 #define     NURS_TOLERANCE_PICK 80
-#define     TOL_AMBIG  1.e-2        /* so close must a point lie to an ambiguous edge to be considered as ambiguous (trimming of nurbs) */
+#define     NURS_ADD_AMBIG_PNTS 20   /* maximum expected points at ambiguous edge */
+#define     TOL_AMBIG  1.e-2        /* so close must a point be to an ambiguous edge to be considered as ambiguous (trimming of nurbs) */
 #define     MAX_PARAM_PER_RECORD 20 /* parameter per record in command files */
 #define     MAX_STACK_PARAMS 30     /* parameter to be written to the stack */
 #define     GL_FEEDBACK_BUFF_SIZE 3000000  /* size of the feedback buffer for nurbs-rendering */
@@ -141,7 +165,8 @@ rendering is done in the index-mode.
                               GLUT_BITMAP_9_BY_15,\
                               GLUT_BITMAP_HELVETICA_18,\
                               GLUT_BITMAP_TIMES_ROMAN_24}
-#define     GLUT_FONT_WIDTH  { 6,7,8,9,10,11 }
+#define     GLUT_FONT_WIDTH  { 5,6,8,9,10,11 }
+#define     GLUT_FONT_HEIGHT  { 16,18,18,18,20,24 }
 #define     DEF_GLUT_FONT      3          /* indx starts with 0 */
 #define     SUM_GLUT_FONTS     6
 
@@ -152,11 +177,11 @@ rendering is done in the index-mode.
 /*
 used fom material illumination
 */
-#define ILLUMINATE_RESULTS 0           /* 1: illumination of displayed results */
+#define ILLUMINATE_RESULTS 1           /* 1: illumination of displayed results */
 
+#define GAMB     0.5
 #define AMB      1.0
 #define DIFF     0.5
-#define DIFF_I     1.
 #define MAT_DIFF 0.6
 #define MAT_SPEC 0.0
 
@@ -186,56 +211,74 @@ used fom material illumination
 /* special sets, generated by cgx  */
 /* internal sets should have a leading "+" or "-" */
 /* Setnames with leading "-" are not shown with qenq */ 
-#define     NSAVE         ":NOSAVE" /* entities not saved when set all is saved */ 
-#define     NJBY          "-NJBY"   /*  */ 
-#define     COPY          "-COPY"   /*  */
-#define     MESH          "-MESH"   /*  */
-#define     NOMESH        "-NOMESH" /*  */
-#define     ZAP           "-ZAP"    /*  */
-#define     IMPC          "-IMPC"   /* independent nodes of equations  */
-#define     DMPC          "-DMPC"   /* dependent nodes of equations    */
-#define     NOMPC         "-NOMPC"  /* not used dependent nodes */
-#define     NOEL          "-NOEL"   /* dependent nodes for which no matching indep-elem was found but it was conn to a close one */
-#define     ORI           "-ORI"    /*  */
-#define     DEP           "-DEP"    /* used for proj */
-#define     IND           "-IND"    /* used for proj  */
-#define     TMP           "-TMP"    /* temporary buffer for several purposes */
-#define     HIGHLIGHT     "-HIGHL"  /* highlights identified entities */
-#define     UORI          "-UORI"   /* save all unoriented items */
+/* Setnames with leading ":" are not shown with prnt */ 
+#define     BLR           "-BLR"    /*  surfaces to be rendered by the structured mesher */
 #define     BNUR          "-BNUR"   /* save all bad nurbs */
 #define     BSUR          "-BSUR"   /* save all bad surfs */
-#define     ETMP          ":ETMP"   /* save all temporary elements, deleted by next call to adjustDrawNodes(1) */
-#define     NOPRJ         "-NOPRJ"  /* failed projection */
 #define     CF            "+CF"     /* contact faces */
+#define     COPY          "-COPY"   /*  */
+#define     DEP           "-DEP"    /* used for proj */
+#define     DMPC          "-DMPC"   /* dependent nodes of equations    */
+#define     EDGES         "+EDGE"   /* all model edge nodes */
+#define     ETMP          ":ETMP"   /* save all temporary elements, deleted by next call to adjustDrawNodes(1) */
+#define     GLUR          "-GLUR"    /* unmeshable surfaces to be rendered by the glu routine */
+#define     HIGHLIGHT     "-HIGHL"  /* highlights identified entities */
+#define     IMPC          "-IMPC"   /* independent nodes of equations  */
+#define     IND           "-IND"    /* used for proj  */
+#define     MESH          "-MESH"   /*  */
+#define     NJBY          "-NJBY"   /* bad elements */ 
+#define     NOEL          "-NOEL"   /* dependent nodes for which no matching indep-elem was found but it was conn to a close one */
+#define     NOMESH        "-NOMESH" /* failed surfaces during 2d meshing */
+#define     NOMPC         "-NOMPC"  /* not used dependent nodes */
+#define     NOPRJ         "-NOPRJ"  /* failed projection */
+#define     NSAVE         ":NOSAVE" /* entities not saved when set all is saved */ 
+#define     ORI           "-ORI"    /*  */
+#define     PENETRATION   "+PENTR"  /* stores nodes of one mesh which penetrate a second mesh */
 #define     PLOT2D        "+PLT2D"  /* selected nodes for the menu-graph (2d-plot) */
 #define     THRESHOLD     "+THRS"   /* nodes with a value above or below the threshold */
+#define     TMP           "-TMP"    /* temporary buffer for several purposes */
+#define     UORI          "-UORI"   /* save all unoriented items */
+#define     WARNINGS      "-WARN"   /* problematic surfaces during 2d meshing */
+#define     ZAP           "-ZAP"    /* entities to be deleted */
 
 typedef struct {
-  char nsave[MAX_LINE_LENGTH];
-  char njby[MAX_LINE_LENGTH];
-  char copy[MAX_LINE_LENGTH];
-  char mesh[MAX_LINE_LENGTH];
-  char nomesh[MAX_LINE_LENGTH];
-  char zap[MAX_LINE_LENGTH];
-  char impc[MAX_LINE_LENGTH];
-  char mpc[MAX_LINE_LENGTH];
-  char nompc[MAX_LINE_LENGTH];
-  char noel[MAX_LINE_LENGTH];
-  char ori[MAX_LINE_LENGTH];
-  char dep[MAX_LINE_LENGTH];
-  char ind[MAX_LINE_LENGTH];
-  char tmp[MAX_LINE_LENGTH];
-  char etmp[MAX_LINE_LENGTH];
-  char highl[MAX_LINE_LENGTH];
-  char uori[MAX_LINE_LENGTH];
+  char blr[MAX_LINE_LENGTH];
   char bnur[MAX_LINE_LENGTH];
   char bsur[MAX_LINE_LENGTH];
-  char noprj[MAX_LINE_LENGTH];
   char cf[MAX_LINE_LENGTH];
+  char copy[MAX_LINE_LENGTH];
+  char dep[MAX_LINE_LENGTH];
+  char etmp[MAX_LINE_LENGTH];
+  char edges[MAX_LINE_LENGTH];
+  char glur[MAX_LINE_LENGTH];
+  char highl[MAX_LINE_LENGTH];
+  char impc[MAX_LINE_LENGTH];
+  char ind[MAX_LINE_LENGTH];
+  char mesh[MAX_LINE_LENGTH];
+  char mpc[MAX_LINE_LENGTH];
+  char njby[MAX_LINE_LENGTH];
+  char noel[MAX_LINE_LENGTH];
+  char nomesh[MAX_LINE_LENGTH];
+  char nompc[MAX_LINE_LENGTH];
+  char noprj[MAX_LINE_LENGTH];
+  char nsave[MAX_LINE_LENGTH];
+  char ori[MAX_LINE_LENGTH];
+  char penetration[MAX_LINE_LENGTH];
   char plot2d[MAX_LINE_LENGTH];
   char thrs[MAX_LINE_LENGTH];
+  char tmp[MAX_LINE_LENGTH];
+  char uori[MAX_LINE_LENGTH];
+  char warnings[MAX_LINE_LENGTH];
+  char zap[MAX_LINE_LENGTH];
 } SpecialSet;
 
+typedef struct {
+  char name[MAX_LINE_LENGTH];
+  GLfloat r;
+  GLfloat g;
+  GLfloat b;
+}Entitycol;
+
 typedef struct {
   double r, g, b;
 } Colours;
@@ -366,13 +409,16 @@ void printHash();
 int write2foam(char *setname, int strings, char **string, Summen *anz, Nodes *node, Faces *face, Elements *e_enqire, Sets *set, Datasets *lcase );
 int write2dolfyn(char *setname, int strings, char **string, Summen *anz, Nodes *node, Faces *face, Elements *e_enqire, Sets *set);
 int readFoam(char *datin, Summen *anz, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr );
+int readVtk(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
 int readAnsysList(char *datin, Summen *anz, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr );
+int readDynaList(char *datin, Summen *apre, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr  );
 
 /* writebp.c */
 int writebp(char *setname, Summen *anz, SumGeo *anzGeo, Points *pnt, Sets *set );
 
 /* writefbd.c */
-int writefbd(char *setname, Summen *anz, SumGeo *anzGeo, Points *pnt, Lines *line, Lcmb *lcmb, Gsur *surf, Gbod *body, Nurbs *nurs, Sets *set, Shapes *shape, Values *value );
+int writefbd(char *setname, char *format );
+void writeInit(FILE *handle);
 
 /* XFunktions_mesa.h */
 void calcOffset(void);
@@ -399,17 +445,19 @@ void descalNodes ( int anz_n, Nodes *node, Scale *scale );
 void scalNodes ( int anz_n, Nodes *node, Scale *scale );
 void calcDatasets( int num_olc, Summen *anz, Nodes *node, Datasets *lcase );
 void transformDatasets( int lc, Summen *anz, Nodes *node, Datasets *lcase, char *method, char *axis  );
-void calcAnimation( int anim_steps, double anim_faktor, int *anim_alfa, int halfperiode, int centerNode, Summen *anz, Nodes *node, Elements *e_enqire, Datasets *lcase, int lc, Scale *scale, char surfFlag, double *colNr, int steps );
+void calcAnimation( int anim_steps, double anim_faktor, int *anim_alfa, int halfperiode, int centerNode, Summen *anz, Nodes *node, Elements *e_enqire, Datasets *lcase, int lc_col, int entity, Scale *scale, char surfFlag, double *colNr, int steps );
 void calcSequence( DsSequence dsSequence, double anim_faktor, int halfperiode, int centerNode, Summen *anz, Nodes *node, Elements *e_enqire, Datasets *lcase, Scale *scale, char surfFlag, double *colNr, int steps, int lcase_animList, int dispFlag );
+void generateDataset( Summen *anz, Datasets **ptr_lcase, char *name, int ncomps, double value, char *dataset_text, int analysis_type, int step_number, char *analysis_name);
+int defineEntity( Summen *anz, Datasets *lcase, Nodes *node, char *name, int ncomp, int ictype, int row, int column);
 
 /* defineEntity.h */
 int hashAlias( SumAsci *sumAsci, char *name, int nr);
 void operateAlias( char *name, char *type );
 void delNod( int anzn, int *number );
 int  nod( Summen *anz, Nodes **nptr, int flag, int nodnr, double x, double y, double z, int scalFlag );
-void pre_nod( char *record);
+int pre_nod( char *record);
 void delElem( int anze, int *index );
-int elem_define( int elnr, int type, int *node, int setFlag, int eattr );
+int elem_define( Summen *anz, Elements **eptr, int elnr, int type, int *node, int setFlag, int eattr );
 int hashPoint( SumAsci *sumAsci, char *name, int nr);
 int hashLine( SumAsci *sumAsci, char *name, int nr);
 int hashLcmb( SumAsci *sumAsci, char *name, int nr);
@@ -435,7 +483,7 @@ int pre_line( char *record, int addFlag);
 void delLcmb( int anz, int *number );
 int lcmb_i( char *name, int add, int anz_l, char  *ori, int *lin );
 int  lcmb_( char *name, int add, int anz_l, char *ori, char *lin );
-void pre_lcmb( char *record, int addFlag);
+int pre_lcmb( char *record, int addFlag);
 void bak_delSurf( int anz, int *number );
 void delSurf( int anz, int *number );
 int  surface_i( char *name, char ori, int blend, int anz_c, char *cori, int *edge, char *typ );
@@ -463,13 +511,8 @@ void drawModelEdges( GLuint list_model_edges, int color, double width,
 void drawDispListEdges( GLuint list, int color, double width, char key, Nodes *node );
 void drawFaces_edge( int num, int *name, Nodes *node, Faces *face, int col, char type );
 void drawElem_edge( int num, int *name, Nodes *node, Elements *e_enqire, int col, char type );
-#if INX_MODE
-void drawDispList( GLuint list, char key, Nodes *node, int *colNr );
-#endif
-#if TEX_MODE
 void drawDispList( GLuint list, char key, Nodes *node, double *colNr );
-#endif
-
+void DrawCommandLine(char *string, int curpos);
 
 
 
@@ -484,9 +527,10 @@ int askGLError(char buffer[134]);
 void defineColIndexes_light(void);
 void defineColIndexes_load(void);
 void defineColIndexes_set(void);
-void defineColTextur_load(void);
+void defineColTextur_load(float alpha);
 void zoom(double xcur0, double ycur0, double xcur1, double ycur1);
 void qzoom(void);
+void initModel(char *string);
 void center(double x, double y, double z);
 void qcenter(int x, int y);
 void qsequence(int x, int y);
@@ -497,8 +541,10 @@ void rot_c(double a);
 void rot_x(double a);
 void rot_y(double a);
 void rot_z(double a);
+int rot_norm(int nr);
 void pre_menu( char *record);
 void pre_rot( char *record);
+int plotNode(int nr);
 void transformResults( char *record);
 int transform( char *record, int anz_n, Nodes *node );
 int calcLineDiv(Lines *line, int nr, double gtol_cos_a, double lmax, double lmin);
@@ -507,26 +553,19 @@ void repNurl(int nr );
 void repNurs(int nr );
 void untrimNurs(int nr );
 int repSurf(int nr, int renderFlag);
-void repShape(int nr );
+int repShape(int nr );
 int repLine( int j );
 void fixMidsideNodes( char *setname, char *parameter);
+void posMidsideNodes(Nodes *node);
 void adjustDrawNodes(int flag);
-void addDispToNodes(Summen *anz, Nodes *n_anim, Nodes *node, Elements *e_enqire, Datasets *lcase, int lc, double amplitude);
-int write2stack(int n, char parameter[20][MAX_LINE_LENGTH]);
+int write2stack(int n, char **parameter);
 int enquireEntities(char *string);
-int write2stack(int n, char parameter[20][MAX_LINE_LENGTH]);
 
 
 /* cgx.h */
-void recalcScale(double val, char flag);
-#if INX_MODE
-void nodalDataset( int cur_entity, int lc, Summen *anz, Scale *scale, Nodes *node
-		   , Datasets *lcase int *colNr, int offset, int maxIndex, int steps, int scalaFlag );
-#endif 
-#if TEX_MODE
+void DrawCommandLine(char *string, int curpos);
 void nodalDataset( int cur_entity, int lc, Summen *anz, Scale *scale, Nodes *node
 		   , Datasets *lcase, double *colNr, int scalaFlag );
-#endif 
 void elementDataset( int auswahl, int lc, Summen *anz, Scale *scale, Datasets *lcase, int offset, int maxIndex, int steps );
 void realloc_colNr(void);
 void entryfunktion( int state );
@@ -536,8 +575,9 @@ void MouseState( int button, int state, int x, int y );
 void frame(void);
 void frameSet(int setNrbuf);
 void menu( int selection );
-void createHardcopy( int selection );
+void createHardcopy( int selection, char *fileName );
 void updateDispLists(void);
+void set_cur_lc(int lc);
 void ConfigureAndShowWindow_Light(void);
 void ConfigureAndShowWindow_Load( void );
 void ConfigureAndShowWindow_Sequence(int dispFlag );
@@ -558,10 +598,14 @@ void createNewMainMenu(void);
 void recompileEntitiesInMenu(int pre_lc);
 void addDispToCoordinates( Nodes *nodes);
 void selectView( int selection );
+int createUSERparam(char *key, char *text);
+int createDSparam(int lc, char *key, char *text);
+int parser( char gkey, char *record, int *curshft, int commandLineFlag);
 void selectData( char *record);
 void specialKeyboard( int gkey, int x, int y );
 void Keyboard( unsigned char gkey, int x, int y );
 void moveModel(void);
+void DrawCommandLine(char *string, int curpos);
 void DrawGraficLoad( void );
 void DrawGraficLight( void );
 void DrawGraficAnimate( void );
@@ -570,15 +614,20 @@ void DrawGraficVector( void );
 void DrawPickedItems(void);
 void drawSets(int mode);
 void DrawAxes();
+void updCommandLine(void);
+void DrawCommandLine(char *string, int curpos);
 void iniDrawMenu(void);
 void DrawMenuLoad( void );
 void DrawMenuSequence( void );
 void DrawMenuLight( void );
 void DrawMenuAnimate( void );
 void DrawMenuSet( void );
+void updDrawingCube( void );
+void setWindowSize(char *string);
+void setWindowPos(char *string);
 void reshape( int width, int height );
-void initLightAndMaterial_rgb( void );
-void initLightAndMaterial_index( void );
+void initLight_rgb( void );
+void pre_subm( char *record);
 
 /* iniMeshData.h     */
 void vdsort ( int *a );
@@ -637,6 +686,7 @@ void qdel( void );
 void qrem( char *record);
 void qpnt( char *record);
 void qlin( char *record);
+void qmov( char *record);
 void qsur( char *record);
 void qbod( char *record);
 void qseq( char *record);
@@ -646,15 +696,16 @@ void qtxt(void);
 void moveText(int n, int x, int y);
 void moveNode(int n, int x, int y);
 void movePoint(GLuint *p, int x, int y);
+void moveSet(GLuint *p, int x, int y);
 int createFillet(int lin, double filletRadius);
 int createElem(int n);
 int createText( int nodenr, int x, int y );
 int createPoint( int x, int y );
 int normalLine(char *name, int *pnts, double l);
 int createLine( char *pnt, int flag );
-int splitSurf( int lmaster, int smaster );
+int splitSurf( int nl, int *ptr_lmaster, int smaster, int projFlag );
 int combineSurfs( int setNr );
-void convertLCMB( int lcmb );
+int convertLCMB( int lcmb );
 void convertLine( int line, int pickbuf );
 int intersect(int lin);
 int qsplitLine( int lin, int x, int y  );
@@ -675,6 +726,7 @@ void sendCflux( char *setname, char *format, Summen *anz, Nodes *node, Elements
 void sendDflux( char *setname, char *format, Summen *anz, Nodes *node, Elements *e_enqire, char *val1, char *val2, char *val3, char type );
 void sendSPC( char *setname, char *format, Summen *anz, Nodes *node, Elements *e_enqire, char *value, char *val1, char *val2, char *val3 );
 void sendSPCF( char *setname, char *format, Summen *anz, Nodes *node, Elements *e_enqire, char *value, char *val1, char *val2, char *val3 );
+void sendSKV( char *setname, char *format, Summen *anz, Nodes *node, Elements *e_enqire, char *val1, char *val2, char *val3 );
 void getNodeNormalen(int **sum_n, Nodes **norm, int setNr, Summen *anz, Faces *face);
 void sendSliders( char *setname, char *format, Summen *anz, Nodes *node, Elements *e_enqire, char *spcType );
 void sendSlidersf( char *setname, char *format, char *spcType );
@@ -696,9 +748,9 @@ void drawBodys_plot( int num, int *name, Gbod *body, Gsur *surf, Lcmb *lcmb, Lin
 void drawNurl_plot( int num, int *name, int col, char type, int width, int pickflag);
 void drawNurs_plot( int num, int *name, int col, char type, int pickflag);
 void drawShapes_plot( int num, int *name, Shapes *shape, Points *point, int col, char type);
-void setLightAndMaterial_rgb(int col);
-int generateTetFromBody(int nr, double teth);
-int generateTetFromSet(int setNr, double teth);
+void setMaterial_rgb(int col, float alpha);
+int generateTetFromBody(int nr, double teth, int eattr, int mesherFlag);
+int generateTetFromSet(int setNr, double teth, int eattr, int mesherFlag);
 int getSetNr(char *setname);
 int getAliasNr(char *name);
 int getNodNr(Summen *anz,Nodes *node,int nodnr );
@@ -734,7 +786,7 @@ int genSplitTrias(int trgtNr, Tri **ptr_tri, int flag );
 void pre_split( char *record);
 void pre_proj( char *record);
 void pre_proc( char *record);
-int splitLine(int l, int *l_nr, int ps_nr, double ps_lbez);
+int splitLine(int l, int *l_nr, int ps_nr);
 void  generateSetIndexes();
 int prnt(char *record);
 int  zap(char *record);
@@ -744,32 +796,36 @@ char getLetter(int i);
 int getFamName( int n, char *c );
 int getNewName( char *name, char *type );
 void pre_movie(char *string);
-int commandoInterpreter( char *type, char *string, int na, int nb, FILE *handle1, int addFlag, int *flag );
-void readfbd( char *datin, int addFlag );
+int commandoInterpreter( char *type, char **ptr_string, int na, int nb, FILE *handle1, int addFlag, int *flag );
+int getCommandLine(FILE *handle1, char **string1);
+int readfbd( char *datin, int addFlag );
 void readfrdfile( char *datin, char *setname );
-void pre_read( char *record );
+int pre_read( char *record );
 void createSuperSets(void);
 void pre_elty( char *record );
 void pre_eqal( char *record );
-void pre_mesh( char *record );
+int pre_mesh( char *record );
+int completeSet_Faces( int setNr, int setNrbuf, int *fUsed, int flag);
 int completeSet_Mesh( int setNr, int setNrbuf, int *elUsed, int flag);
 int completeSet_Lines( int setNr, int setNrbuf, int *elUsed, int flag);
 int completeSet_frame( int setNr, char type);
 int completeSet( char *setname, char *type );
+int separateMeshes( char *setName, char *grpName);
 int sendTriangles(  char *setname );
 int write2netgenEdges(char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
 void pre_write( char *record );
 void senddata( char *setname, char *format, Summen *anz, Nodes *node, Elements *e_enqire, Datasets *lcase , Scale *scale, int complete, char **val );
 int calcCoefficientsTet(int nslav, int *emas, int n_closest_tets, Nodes *node, Tetraeder *tet, double cof[4], int extrapolflag);
 int interpol3d(int set1, int set2, char *format, char *dataset, int extrapolflag);
+void addCommandToInitFile(char *string);
 
 /* sendMpc.c */
 void sendMpc( char *setname, char *format, char *rotation, double *vector  );
 void cycmpc(int set1, int set2, char *format, char *value, char *corr);
-void areampc(int set1, int set2, char *format, char *type, char *value, char *corr, int offset, Nodes *nptr, int extrapolflag);
+void areampc(int set1, int set2, char *format, char *type, char *value, char *corr, int offset, Nodes *nptr, int extrapolflag, int scalFlag);
 void gap(char *record);
 int calcCoefficientsTri(  Nodes *node, int n0, CTri3 *ctri3, int  e, double *dist, double *c, double *n);
-double find_tri3( Nodes *node, int n, CTri3 *ctri3, int e);
+double check_tri3( double *pp, double *pt1, double *pt2, double *pt3, int tolflag );
 void corrNode(  Nodes *node, CTri3 *ctri3, int  e, double g, int n, double *vNneu );
 void createMPCs();
 
@@ -785,7 +841,7 @@ typedef struct {
 /* graph.c */
 void param2D(char *par1, int *dsNr, char *par2 );
 void plot2D(char *type, int setNr, int *dsNr, int entity );
-void graph( char *record);
+int  graph( char *record);
 
 /* corrad.c */
 void corrad( int setNr, double minradius);
@@ -799,59 +855,61 @@ int write2tochnog(char *datout, Summen *anz, Nodes *node, Elements *elem, Datase
 /* readccx.c */
 void delSetx( char *setname);
 int pre_setax( char *setname, char *type, char *name);
-int readccx(char *datin, Summen *anz, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr);
+int readccx(char *datin, Summen *anz, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr, int checkflag);
 int getFilePointer( int filesopen, FILE **openfile, char *rec_str);
 
 
-double repairNurbs( Nurbs *nurbs, int nr, int deg, int dir);
-int evalNurbsWithNormalVector( Nurbs *nurbs, int nr, int sum_p, double *pnt_u, double *pnt_v, Points *pnt, Points *nv);
-int evalNurbs( Nurbs *nurbs, int nr, int sum_p, double *pnt_u, double *pnt_v, Points *pnt);
+double repairNurbs( int nr, int deg, int dir);
+int evalNurbsWithNormalVector( int nr, int sum_p, double *pnt_u, double *pnt_v, Points *pnt, Points *nv);
+int evalNurbs( int nr, int sum_p, double *pnt_u, double *pnt_v, Points *pnt);
 void cppfreearray(double *array);
-int createBlendedNurbs(int nr, Points **pntpntr, Lines *line, Lcmb *lcmb, Gsur *surf );
-void projSurfToNurbs( Nurbs *nurbs, int nr, Gsur *surf, int snr, Nodes *node );
-void projSetToNurbs( Nurbs *nurbs, int nr, Sets *set, int setNr, Points *pnt);
-double *projPntsToNurbs( Nurbs *nurbs, int nr, int anz_p, Points *pnt);
-double proj1PntToNurbs( Nurbs *nurbs, int nr, double *pnt);
-int trimNurbs( Nurbs *nurbs, int nr, int patch, double);
+int createBlendedNurbs(int nr);
+void projSurfToNurbs( int nr, Gsur *surf, int snr, Nodes **node );
+void projSetToNurbs( int nr, Sets *set, int setNr, Points *pnt, Nodes **node);
+double *projPntsToNurbs( int nr, int anz_p, Points *pnt);
+double proj1PntToNurbs( int nr, double *pnt);
+int trimNurbs( int nr, int patch, double);
 
 
 /* fillBody2.c */
-int  fillSurf_ab( int j, int umax, int vmax, int *n_uv, int amax, int bmax, int *n_ab, int offs1, int offs2, double *x, double *y, double *z );
-int  fillSurf_ba2( int sur, int *div_l, int div_a, int div_b, int sa1, int sa2, int sb1, int sb2, int *n_uv, int *umax, int *vmax, int *n_ba, int *amax, int *bmax,  int *offs_sa1, int *offs_sa2, double *x, double *y, double *z, double *x_ba, double *y_ba, double *z_ba );
-int fillBody2( int b_indx, int *srefp, int **div_l, int *unbalance, int **nabc, int *amax, int *bmax, int *cmax, int *offs_sa1, int *offs_sa2, int *msur, int *div_sa1, int *div_sa2 );
+int evalBody2( int b_indx );
+int fillBody2( int b_indx, int **pn_abc );
 
 
 /* meshSet.c */
+void copyThreadMesh(int nthreads);
 void cornerNodes( int vmax, int umax, int j, int *n_uv );
 int  newCornerPositions( int n, int u,int v,int umax, int vmax, double *lx,double *ly,double *lz,
  double *x,double *y,double *z, int flag );
-int  fillSurf( int j, int *n_uv, int umax, int vmax, double *x, double *y, double *z );
-int  fillSurf2( int sur, int *div_l, int div_a, int div_b, int sa1, int sa2, int sb1, int sb2,
+int  fillSurf(int j, int *n_uv, int umax, int vmax, double *x, double *y, double *z );
+int  fillSurf2(int sur, int *div_l, int div_a, int div_b, int sa1, int sa2, int sb1, int sb2,
  int *n_uv, int *umax, int *vmax, int *n_ba, int *amax, int *bmax, int *offs_sa1, int *offs_sa2,
  double *x, double *y, double *z );
 void edgeNodes( int vmax, int umax, int j, int *n_uv );
 int  newEdgePositions( int n, int u,int v,int umax, int vmax, double *lx,double *ly,double *lz, double *x,double *y,double *z, int flag );
 
-double calcMeshDistance( Nurbs *nurbs, int nr, int np, Points *ppre, int e, Points *pcg, int *tri);
-int triSplitter( Nurbs *nurbs, int nr, int *ptr_np,  Points **ptr_ppre, int *ptr_ne, int **ptr_tri, int **ptr_pntflag, int **edge_n2, int *pnt_indx, double **ptr_pntu, double **ptr_pntv, double max_ratio, double *tab_u, double *tab_lu, double *tab_bu, double *tab_v, double *tab_lv, double *tab_bv);
+double calcMeshDistance( int nr, int np, Points *ppre, int e, Points *pcg, int *tri);
+int triSplitter( int nr, int *ptr_np,  Points **ptr_ppre, int *ptr_ne, int **ptr_tri, int **ptr_pntflag, int **edge_n2, int *pnt_indx, double **ptr_pntu, double **ptr_pntv, double max_ratio, double *tab_u, double *tab_lu, double *tab_bu, double *tab_v, double *tab_lv, double *tab_bv);
 int checkMesh(int np, Points *ppre, int e, int *tri);
-int meshPoints( int setNr );
+int meshPoints( int setNr, int renderFlag );
 int straightNodes( int j, int k, int d, double *pn );
 int arcNodes( int j, int k, int d, double *pn );
 int splineNodes( int j, int k, int d, double *pn );
 int nurlNodes( int j, int k, int d, double *pn );
-int meshLines( int setNr );
+int meshLines( int setNr, int renderFlag );
 int newDivisions( int *div_l, int *div_a,int *div_b, int *sa1,int *sa2,int *sb1,int *sb2 );
 int addTwoLines( int l1, char o1, char typ1, int l2, char o2, char typ2 );
 int splitLineAtDivratio(int edge, int typ, double splitdiv, int *lnew, char  *typnew);
-int mesh_tr3u(int nr );
+int mesh_tr3u(int nr, int renderFlag);
 int merge_nlocal( Rsort *rsort, Nodes *nlocal, Elements *ctri3, int anz_e, int anz_n, int indx, int lock, double tol );
 int deleteFreeEdges(Nodes *node, int *snods, int anz_n, Elements *elems, int anz_e );
 int mesh_tr3g( int s );
 int determineBestCorners( int s, int *cl);
-void *meshSurf( void *arg);
-int meshSurfs( int setNr );
-int fillBody( int b, int *srefp, int umax, int vmax, int wmax, int *n_uvw, double *x, double *y, double *z );
+int substituteSurf(int j);
+int meshSurf( int anz_s, int sindx, int meshflag, int renderFlag);
+int meshSurfs( int setNr, int renderFlag );
+int evalBody( int b_indx, int *srefp );
+int fillBody( int b_indx, int **pn_uvw);
 int getSurfDivs( int b, int s, int *div_l );
 int meshImprover( int *etyp, int *nodes, int *elements, int *n_indx, int *n_ori, double **n_coord, int **e_nod, int *n_type, int **n_corner, int **s_div );
 int bodyFrom5Surfs( int *b);
@@ -866,7 +924,7 @@ int surfMesh(int *imax, int *jmax, double *x, double *y, double *z);
 /*  bodyMesh2.h */
 int bodyMesh2(int *wmax, int *vmax, int *umax, double *x, double *y, double * z);
 
-/*  improveMesh.o */
+/*  improveMesh.h */
 int smooth_length( double *co, int *nk, int *kon, int *ne, int *iptr1, int *iptr2, int *nside, int *ineino, int *nkind, int *neigh, int *ndiv, double *pneigh, int *maxsumdiv, int *etyp, int *nori);
 int smooth_angle( double *co, int *nk, int *kon, int *ne, int *iptr1, int *iptr2, int *nside, int *ineino, int *nkind, int *neigh, int *ndiv, double *pneigh, int *maxsumdiv, int *etyp, int *nori);
 int smooth_midnodes( double *co, int *nk, int *kon, int *ne, int *iptr1, int *iptr2, int *nside, int *ineino, int *nkind, int *neigh, int *ndiv, double *pneigh, int *maxsumdiv, int *etyp, int *nori);
@@ -887,9 +945,9 @@ int calcBadElements(char *setname);
 
 /* contact.c */
 int mergeCluster(Cluster *cluster, int c1, int c2);
-int getMeshSections(int setNr, Faces *face, Nodes *node);
-int getDepNodes( Summen *anz, int set1, int set2, Elements *e_enqire, Faces *face, Nodes *node, double tol);
-int getFacePair( Summen *anz, int set1, int set2, Elements *e_enqire, Faces *face, Nodes *node, double tol, int *mpcset);
+int getMeshSections(int setNr, Summen *anz, Faces *face, Nodes *node);
+int getDepNodes( Summen *anz, int set1, int set2, Elements *e_enqire, Faces *face, Nodes *node, double tol, int *sum_n, int **nodesets);
+int getFacePair( Summen *anz, int set1, int set2, Elements *e_enqire, Faces *face, Nodes *node, double tol, int *mpcset, char *action);
 
 /* makeTriFromElems.c */
 int makeTriFromElems(int setNr, int eset, int nmax, Sets *set, Elements *elem,  CTri3 **pctri3, int ***ptri3_index, int **pntri_nodes);
@@ -899,8 +957,9 @@ int readstep( char *datin, int addFlag );
 void copyDatasetToNodes(Summen *anz, Nodes *node, Datasets *lcase, int lc, CopiedNodeSets copiedNodeSets);
 int pre_readfrdblock( CopiedNodeSets *copiedNodeSets, int lc, Summen *anz, Nodes *node, Datasets *lcase );
 int surfToNurs(int s);
-int coneToNurs(int s);
-int sphToNurs(int s);
+int torusToNurs(int s, int flag);
+int coneToNurs(int s, int flag);
+int sphToNurs(int s, int flag);
 int surfToShape(int s);
 void uncut(int surfFlagBuffer);
 int crecord( char *rec_str, char **dat);
@@ -908,3 +967,14 @@ void descalAll(void);
 int getSetNrx(char *name);
 int setax( int setNr, char *type, int number);
 int pre_setax( char *string, char *type, char *name);
+void pre_repSurf( int setNr);
+void *thread_repSurf( void *vargp);
+double calcGTOL(int setNr);
+int fillNurbsSurf(int nurbsnr, int nr);
+void moveLineEndPoint(int lineNr, int pntNr, double llength);
+int calcTrimLoops(int nurbsnr, int nr);
+int check_lineLoop(double *cg, int np, double *pnt_u, double *pnt_v, int e, int *tri);
+void mergeElem( int setNr, int delFlag );
+void DrawCommandLine(char *string, int curpos);
+void compileTextString(Texts *ntext, int nr);
+int readDynaList(char *datin, Summen *apre, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr  );
