@@ -37,6 +37,7 @@ extern double     gtol;
 extern Scale     scale[1];
 extern Summen    anz[1];
 extern Nodes     *node;
+extern Faces     *face;
 extern Elements  *e_enqire;
 extern Datasets *lcase;
 
@@ -86,17 +87,17 @@ int createCenterpnt( char *pkt1, char *pkt2, char *buf, char *pkt3, char reuseFl
   if(pnr2>-1)
   {
     fi = atof(buf)*PI/180.;
-    if (pnr1==-1)
+    if (pnr1<0)
     {
       errMsg(" Point1 (%s) is undefined\n", pkt1 );
       return(-1);
     }
-    if (pnr2==-1)
+    if (pnr2<0)
     {
       errMsg(" Point2 (%s) is undefined\n", pkt2 );
       return(-1);
     }
-    if (pnr3==-1)
+    if (pnr3<0)
     {
       errMsg(" Point3 (%s) is undefined\n", pkt3 );
       return(-1);
@@ -327,6 +328,7 @@ void copyDatasetToNodes(Summen *anz, Nodes *node, Datasets *lcase, int lc, Copie
           break;
         }
       }
+    runAsIfND0:;
       if(nd==0) /* single results */
       {
         if(lcase[lc].ictype[0]==2) /* vector */
@@ -453,7 +455,8 @@ void copyDatasetToNodes(Summen *anz, Nodes *node, Datasets *lcase, int lc, Copie
         stl=-sin(theta*-sign);
 
         /* its harmonic, search the second result */
-        lc1=lc2=lc;
+        lc1=lc;
+        lc2=-1;
         //if(lc) { i=lc-1; while((lcase[i].step_number==lcase[lc].step_number)&&(i>=0)) i--; i++; }
         if(lc) { for (i=lc-1; i>=0; i--) { if(lcase[i].step_number!=lcase[lc].step_number) break; } i++; }
         else i=1;
@@ -486,6 +489,13 @@ void copyDatasetToNodes(Summen *anz, Nodes *node, Datasets *lcase, int lc, Copie
 
         if(lcase[lc1].ictype[0]==2) /* vector */
         {
+          if(lc2==-1)
+	  {
+            printf(" WARNING: Found no imaginary part for dataset:%d. This dataset will be treated as valid for nd=0,\n",lc1+1);
+            nd=0;
+            goto runAsIfND0;
+	  }
+
           rectcyl(2, csab, anz->n, node, lcase, lc1, 'd');
           rectcyl(2, csab, anz->n, node, lcase, lc2, 'd');
           ncomps=3;
@@ -509,6 +519,13 @@ void copyDatasetToNodes(Summen *anz, Nodes *node, Datasets *lcase, int lc, Copie
         }
         else if(lcase[lc1].ictype[0]==4) /* matrix */
         {
+          if(lc2==-1)
+	  {
+            printf(" WARNING: Found no imaginary part for dataset:%d. This dataset will be treated as valid for nd=0,\n",lc1+1);
+            nd=0;
+            goto runAsIfND0;
+	  }
+
           rectcyl(2, csab, anz->n, node, lcase, lc1, 's');
           rectcyl(2, csab, anz->n, node, lcase, lc2, 's');
           ncomps=6;
@@ -600,7 +617,7 @@ void copyDatasetToNodes(Summen *anz, Nodes *node, Datasets *lcase, int lc, Copie
             }
           }
 	}
-        if(lc1!=lc2)
+        if((lc2!=-1)&&(lc1!=lc2))
         {
           if(lc==lc1) i=lc2; else i=lc1;
           calcDatasets( i, anz, node, lcase );
@@ -629,6 +646,8 @@ void copyDatasetToNodes(Summen *anz, Nodes *node, Datasets *lcase, int lc, Copie
   }
 }
 
+
+
 /*------------------------------------------------------------------*/
 /* copy                                                             */
 /*------------------------------------------------------------------*/
@@ -638,15 +657,17 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
   static int *lines=NULL;
 
   int   i,j,k,p,l,s,b,p1,p2,p3,p4,p5,p6,p7, se,nr,S=-1, transtyp=-1;
-  int elnr, n, en[26], lc, ipuf;
+  int elnr, n, en[26], lc, ipuf, projFlag=0;
   static Nodes *bufn=NULL;
   char name[MAX_LINE_LENGTH];
   char setname[MAX_LINE_LENGTH];
-  double csab[6], fi=0.;
+  char buffer[MAX_LINE_LENGTH];
+  double csab[6], fi=0.,tol=0;
 
   char  pkt1[MAX_LINE_LENGTH], pkt2[MAX_LINE_LENGTH];
   int   pnr1, pnr2;
 
+
   /* copy nodes */
   if(set[setNr].anz_n>0)
   {
@@ -663,17 +684,19 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     for (i=0; i<j; i++)
     {
       p=set[setNr].node[i];
-      if (p==-1)
+      if (p<0)
       {
         errMsg(" node-nr:%d is undefined\n", set[setNr].node[i] );
         return(-1);
       }
+      bufn[i].indx=p;
       bufn[i].nx=node[p].nx;
       bufn[i].ny=node[p].ny;
       bufn[i].nz=node[p].nz;
     }
     transtyp=transform( trans, set[setNr].anz_n, bufn );
     if (transtyp==-1) return(-1);
+    if (transtyp==7) transtyp=1;
 
     /* create new nodes */
     for (i=0; i<j; i++)
@@ -709,7 +732,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
           pnr1=getPntNr( pkt1 );
           pnr2=getPntNr( pkt2 );
 
-          if (pnr1==-1)
+          if (pnr1<0)
           {
             errMsg(" ERROR: Point1 (%s) is undefined\n", pkt1 );
             return(-1);
@@ -721,7 +744,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
             csab[2] = point[pnr1].pz*scale->w+scale->z;
           }
 
-          if (pnr2==-1)
+          if (pnr2<0)
           {
             csab[3] = point[pnr1].px*scale->w+scale->x;
             csab[4] = point[pnr1].py*scale->w+scale->y;
@@ -795,11 +818,14 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     scalNodes ( anz->n, node, scale );
 
     /* new midnodes for drawing if no elements are to copy */
-    adjustDrawNodes(1);
-    getElemNormalen( e_enqire, node, anz->e );
-    makeSurfaces();
-    realloc_colNr();
-    updateDispLists();
+    //if(set[setNr].anz_e<1)
+    {
+      adjustDrawNodes(1);
+      getElemNormalen( e_enqire, node, anz->e );
+      makeSurfaces();
+      realloc_colNr();
+      updateDispLists();
+    }
   }
 
 
@@ -873,7 +899,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
             for (n=0; n<ipuf; n++) en[n]=dep_n[e_enqire[ elnr ].nod[n]];
 	}
       }
-      elem_define( anz->enext, e_enqire[ elnr ].type, en, 1, e_enqire[ elnr ].attr );
+      elem_define(anz,&e_enqire, anz->enext, e_enqire[ elnr ].type, en, 1, e_enqire[ elnr ].attr );
       dep_e[elnr] = anz->enext; /* remember the index of the new elem */
       seta( settrgt, "e", anz->enext );
  
@@ -905,7 +931,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     for (i=0; i<j; i++)
     {
       p=set[setNr].pnt[i];
-      if (p==-1)
+      if (p<0)
       {
         errMsg(" ERROR: Pointnr:%d is undefined\n", set[setNr].pnt[i] );
         return(-1);
@@ -914,8 +940,22 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
       bufn[i].ny=point[p].py*scale->w+scale->y;
       bufn[i].nz=point[p].pz*scale->w+scale->z;
     }
-    transtyp=transform( trans, set[setNr].anz_p, bufn );
-    if (transtyp==-1) return(-1);
+
+    /* if trans=='nor' use proj */
+    nr=sscanf( trans,"%s %lf", buffer, &fi );
+    if(( compare(buffer,"nor",3) == 3 )||( compare(buffer,"NOR",3) == 3 ))
+    {
+      tol=fi;
+      if(nr==3) sprintf(buffer,"proj %s %s nor %f %f\n", set[settrgt].name, set[setNr].name, fi, tol );
+      else sprintf(buffer,"proj %s %s nor %f\n", set[settrgt].name, set[setNr].name, fi );
+      transtyp=1;
+      projFlag=1;
+    }
+    else
+    {
+      transtyp=transform( trans, set[setNr].anz_p, bufn );
+      if (transtyp==-1) return(-1);
+    }
 
     for (i=0; i<j; i++)
     {
@@ -932,6 +972,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
       /* add to all sets were the orig is a member */
       if(appendSetFlag) for(se=1; se<anz->sets; se++) if((se!=setNr)&&(se!=mastersetNr)&&(set[se].name!=(char *)NULL)&&(set[se].name[0]!='-'))  if(!set[se].type) if( getIndex(&set[se].pnt, set[se].anz_p, set[setNr].pnt[i]) >-1) { seta( se, "p", p );  }
     }
+    if (projFlag==1) {  pre_proj(buffer); }
   }
 
   /* copy the lines */
@@ -941,7 +982,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     for (i=0; i<j; i++)
     {
       l=set[setNr].line[i];
-      if (l==-1)
+      if (l<0)
       {
         errMsg(" Linenr:%d is undefined\n", set[setNr].line[i] );
         return(-1);
@@ -998,7 +1039,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     for (i=0; i<j; i++)
     {
       p=set[setNr].lcmb[i];
-      if (p==-1)
+      if (p<0)
       {
         errMsg(" Lcmbnr:%d is undefined\n", set[setNr].lcmb[i] );
         return(-1);
@@ -1059,6 +1100,8 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
       nurbs[nr].v_nknt= nurbs[S].v_nknt;
       nurbs[nr].u_stride= nurbs[S].u_stride;
       nurbs[nr].v_stride= nurbs[S].v_stride;
+      nurbs[nr].nurbsType=nurbs[S].nurbsType;
+      nurbs[nr].axis=nurbs[S].axis;
 
       if ( (nurbs[nr].uknt = (GLfloat *)malloc( (nurbs[S].u_nknt+1) * sizeof(GLfloat))) == NULL )
         printf("\n\n ERROR: realloc failed uknt\n\n");
@@ -1107,7 +1150,6 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
       nurbs[nr].vstep=NULL;
       nurbs[nr].ustep=NULL;
       nurbs[nr].Nurb = (GLUnurbsObj *)gluNewNurbsRenderer();
-      nurbs[nr].nurbsType=1;
       nurbs[nr].uvflipped=NULL;
       nurbs[nr].sum_ambiguousPnts=NULL;
       for (i=0; i<anz->sets; i++)
@@ -1130,7 +1172,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     j=set[setNr].anz_sh;
     for (i=0; i<j; i++)
     {
-      if (set[setNr].shp[i]==-1)
+      if (set[setNr].shp[i]<0)
       {
         errMsg(" Shapenr:%d is undefined\n", set[setNr].shp[i] );
         return(-1);
@@ -1178,6 +1220,17 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
         if(p1>-1)
           l=shape_i( nurbs[dep_S[p1]].name, shape[set[setNr].shp[i]].type, dep_S[p1], 0,0,0, 0,0,0);
       } 
+      else if(shape[set[setNr].shp[i]].type==5)
+      {
+        l= getNewName( name, "sh" );
+        if ( l == -1 )
+        { printf("copy: could not create new shape\n"); return(-1); }
+        p1=shape[set[setNr].shp[i]].p[0];
+        p2=shape[set[setNr].shp[i]].p[1];
+        p3=shape[set[setNr].shp[i]].p[2];
+        p4=shape[set[setNr].shp[i]].p[3];
+        l=shape_i( name, shape[set[setNr].shp[i]].type, dep_p[p1], dep_p[p2], dep_p[p3], dep_p[p4], 0, 0, 0 );
+      } 
       if ( l <0 )
       { printf("copy: could not create new shape\n"); return(-1); }
       dep_sh[set[setNr].shp[i]] = l; /* remember the index of the new shape */
@@ -1195,7 +1248,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     for (i=0; i<j; i++)
     {
       s=set[setNr].surf[i];
-      if (s==-1)
+      if (s<0)
       {
         errMsg(" surfnr:%d is undefined\n", set[setNr].surf[i] );
         return(-1);
@@ -1218,6 +1271,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
         { printf("copy: could not create new surf\n"); return(-1); }
       surf[s].etyp =surf[set[setNr].surf[i]].etyp;
       surf[s].eattr=surf[set[setNr].surf[i]].eattr;
+      surf[s].elock=surf[set[setNr].surf[i]].elock;
       dep_s[set[setNr].surf[i]] = s; /* remember the index of the new surf */
       seta( settrgt, "s", s );
 
@@ -1233,7 +1287,7 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
     for (i=0; i<j; i++)
     {
       b=set[setNr].body[i];
-      if (b==-1)
+      if (b<0)
       {
         errMsg(" bodynr:%d is undefined\n", set[setNr].body[i] );
         return(-1);
@@ -1259,9 +1313,10 @@ int copy_set( int settrgt, char *trans, int setNr, int *dep_n, int *dep_e, int *
       if(appendSetFlag) for(se=1; se<anz->sets; se++) if((se!=setNr)&&(se!=mastersetNr)&&(set[se].name!=(char *)NULL)&&(set[se].name[0]!='-')) if(!set[se].type) if( getIndex(&set[se].body, set[se].anz_b, set[setNr].body[i]) >-1) { seta( se, "b", b );  }
     }
   }
+
   if ( (bufn = (Nodes *)realloc((Nodes *)bufn, 1 * sizeof(Nodes))) == NULL )
     printf("\n\n ERROR: malloc failed in copy_set()\n\n")  ;
-
+  
   return(transtyp);
 }
 
@@ -1412,6 +1467,23 @@ void pre_swep( char *record )
   {
     seta( setcopy, "n", set[setNr].node[i] );
   }
+  /*  make elements from faces to sweep them to volume elements */
+  for (i=0; i<set[setNr].anz_f; i++)
+  {
+    e=set[setNr].face[i];
+    /* generate elems from volume element faces */
+    if(e_enqire[face[e].elem_nr].type<7)
+    {
+      if(elem_define(anz,&e_enqire, anz->enext, face[e].type, face[e].nod, 0, 0 ) <0 ) errMsg("WARNING: element based on face of elem %d could not be created\n", face[e].elem_nr);
+      else
+      {
+        seta( setNr, "e", anz->enext );
+        anz->enext++;
+      }
+    }
+    // add the face as a possible target for projection purposes in copy_set(), they will not be copied
+    seta( setcopy, "f", e );
+  }
   /*  add all elements */
   for (i=0; i<set[setNr].anz_e; i++)
   {
@@ -1517,14 +1589,14 @@ void pre_swep( char *record )
     {
       j=set[setNtmp].node[i];
       p1=pnt( p1name, node[j].nx, node[j].ny, node[j].nz, 0 );
-      if (p1==-1)
+      if (p1<0)
       {
         errMsg(" point (%d) is undefined\n", p1name );
         goto no_swep;
       }
       n=dep_n[ldiv-1][j];
       p2=pnt( p2name, node[n].nx, node[n].ny, node[n].nz, 0 );
-      if (p2==-1)
+      if (p2<0)
       {
         errMsg(" corresp. point to point (%d) is undefined\n", p2name );
         goto no_swep;
@@ -1601,7 +1673,7 @@ void pre_swep( char *record )
     for (i=0; i<anz_e; i++)
     {
       e=set[setNr].elem[i];
-      if (e==-1)
+      if (e<0)
       {
         errMsg(" elem (%d) is undefined\n", e );
         goto no_swep;
@@ -1620,7 +1692,7 @@ void pre_swep( char *record )
             en[n]=dep_n[k][e_enqire[e].nod[n]];
             en[n+ipuf]=dep_n[k+1][e_enqire[e].nod[n]];
           }
-          elem_define( anz->enext++, 5-ipuf, &en[0], 1, 0 );
+          elem_define(anz,&e_enqire, anz->enext++, 5-ipuf, &en[0], 1, 0 );
 
           /* add to all sets were the orig is a member */
           if(appendSetFlag) for(se=1; se<anz->sets; se++) if((se!=setNr)&&(se!=mastersetNr)&&(set[se].name!=(char *)NULL)&&(set[se].name[0]!='-'))  if(!set[se].type) if( getIndex(&set[se].elem, set[se].anz_e, e) >-1) { seta( se, "e", anz->emax );  }
@@ -1630,7 +1702,7 @@ void pre_swep( char *record )
           en[n]=e_enqire[e].nod[n];
           en[n+ipuf]=dep_n[0][e_enqire[e].nod[n]];
         }
-        elem_define( e, 5-ipuf, &en[0], 0, 0 );
+        elem_define(anz,&e_enqire, e, 5-ipuf, &en[0], 0, 0 );
       }
       ipuf=0;
       if (e_enqire[e].type == 8) ipuf = 6;  /* TRI6 */
@@ -1652,8 +1724,8 @@ void pre_swep( char *record )
             //node[dep_n[k+1][e_enqire[e].nod[n]]].pflag=1;
             delNod(1,&dep_n[k+1][e_enqire[e].nod[n]]);
           }
-          if(ipuf==6) elem_define( anz->enext++, 5, &en[0], 1, 0 );
-          if(ipuf==8) elem_define( anz->enext++, 4, &en[0], 1, 0 );
+          if(ipuf==6) elem_define(anz,&e_enqire, anz->enext++, 5, &en[0], 1, 0 );
+          if(ipuf==8) elem_define(anz,&e_enqire, anz->enext++, 4, &en[0], 1, 0 );
 
           /* add to all sets were the orig is a member */
           if(appendSetFlag) for(se=1; se<anz->sets; se++) if((se!=setNr)&&(se!=mastersetNr)&&(set[se].name!=(char *)NULL)&&(set[se].name[0]!='-'))  if(!set[se].type) if( getIndex(&set[se].elem, set[se].anz_e, e) >-1) { seta( se, "e", anz->emax );  }
@@ -1671,8 +1743,8 @@ void pre_swep( char *record )
           //node[dep_n[0][e_enqire[e].nod[n]]].pflag=1;
           delNod(1,&dep_n[0][e_enqire[e].nod[n]]);
         }
-        if(ipuf==6) elem_define( e, 5, &en[0], 0, 0 );
-        if(ipuf==8) elem_define( e, 4, &en[0], 0, 0 );
+        if(ipuf==6) elem_define(anz,&e_enqire, e, 5, &en[0], 0, 0 );
+        if(ipuf==8) elem_define(anz,&e_enqire, e, 4, &en[0], 0, 0 );
       }
     }
     /* the nodes numbers have to be stored before the nodes for drawing purposes are generated */
@@ -1699,13 +1771,13 @@ void pre_swep( char *record )
     for (i=0; i<j; i++)
     {
       p1=set[setNr].pnt[i];
-      if (p1==-1)
+      if (p1<0)
       {
         errMsg(" point (%d) is undefined\n", set[setNr].pnt[i] );
         goto no_swep;
       }
       p2=dep_p[set[setNr].pnt[i]];
-      if (p2==-1)
+      if (p2<0)
       {
         errMsg(" corresp. point to point (%d) is undefined\n", set[setNr].pnt[i] );
         goto no_swep;
@@ -1760,13 +1832,13 @@ void pre_swep( char *record )
     for (i=0; i<j; i++)
     {
       lines[0]=set[setNr].line[i];
-      if (lines[0]==-1)
+      if (lines[0]<0)
       {
         errMsg(" line (%d) is undefined\n", set[setNr].line[i] );
         goto no_swep;
       }
       lines[1]=dep_l[set[setNr].line[i]];
-      if (lines[1]==-1)
+      if (lines[1]<0)
       {
         errMsg(" dep line from line (%d) is undefined\n", set[setNr].line[i] );
         goto no_swep;
@@ -1901,15 +1973,9 @@ void pre_swep( char *record )
 	//*****************************************************//
 	// 	generate and initialise a nurs-object          //
 	//*****************************************************//
-	char * lineBuffer = NULL;//char buffers
-	char * nurbsName = NULL;
-        
-        if((lineBuffer = (char *)realloc(lineBuffer,70*sizeof(char))) == NULL )
-        { printf(" ERROR: realloc failure\n\n"); }  
-        
-        if((nurbsName = (char *)realloc(nurbsName,10*sizeof(char))) == NULL )
-        { printf(" ERROR: realloc failure\n\n"); }    
-        
+	char lineBuffer[MAX_LINE_LENGTH];//char buffers
+	char nurbsName[MAX_LINE_LENGTH];
+               
 	getNewName(nurbsName,"S");
 	//set header line of NURBS surface
 	sprintf(lineBuffer,"%s DEFINE COMPACT %d %d %d %d %d %d",nurbsName, mySurf.uDeg, mySurf.vDeg, mySurf.nUPol, mySurf.nVPol, mySurf.nUKnt, mySurf.nVKnt);
@@ -1994,14 +2060,14 @@ void pre_swep( char *record )
     {
       lines[0]=set[setNr].lcmb[i];
       types[0]='c';
-      if (lines[0]==-1)
+      if (lines[0]<0)
       {
         errMsg(" lcmb (%d) is undefined\n", set[setNr].lcmb[i] );
         goto no_swep;
       }
       lines[1]=dep_c[set[setNr].lcmb[i]];
       types[1]='c';
-      if (lines[1]==-1)
+      if (lines[1]<0)
       {
         errMsg(" dep lcmb from lcmb (%d) is undefined\n", set[setNr].lcmb[i] );
         goto no_swep;
@@ -2175,8 +2241,8 @@ void pre_swep( char *record )
 	//*****************************************************//
 	// 	generate and initialise an nurs-object         //
 	//*****************************************************//
-	char lineBuffer[50];//char buffers
-	char nurbsName[10];
+	char lineBuffer[MAX_LINE_LENGTH];//char buffers
+	char nurbsName[MAX_LINE_LENGTH];
 	getNewName(nurbsName,"S");
 	//set header line of NURBS surface
 	//sprintf(name,"N%c%c%c",lcmb[lines[0]].name[1],lcmb[lines[0]].name[2],lcmb[lines[0]].name[3]);
@@ -2257,7 +2323,7 @@ void pre_swep( char *record )
       { printf("ERROR: realloc failure in pre_swep()\n"); return; }
 
       s[0]=set[setNr].surf[i];
-      if (s[0]==-1)
+      if (s[0]<0)
       {
         errMsg(" surf (%d) is undefined\n", set[setNr].surf[i] );
         goto no_swep;
@@ -2267,7 +2333,7 @@ void pre_swep( char *record )
       surf[s[0]].ne=0;
 
       s[1]=dep_s[set[setNr].surf[i]];
-      if (s[1]==-1)
+      if (s[1]<0)
       {
         errMsg(" dep surf from surf (%d) is undefined\n", set[setNr].surf[i] );
         goto no_swep;
@@ -2323,7 +2389,7 @@ void pre_swep( char *record )
 
 void pre_copy( char *record )
 {
-  int  length, i, setNr, settrgt, setcopy;
+  int  length, i,e, setNr, settrgt, setcopy;
   char setname[MAX_LINE_LENGTH], targetnam[MAX_LINE_LENGTH];
   static int   *dep_n, *dep_e, *dep_p, *dep_l, *dep_c, *dep_s, *dep_se, *dep_sh, *dep_S;
   int  appendSetFlag=0;
@@ -2412,6 +2478,13 @@ void pre_copy( char *record )
   {
     seta( setcopy, "n", set[setNr].node[i] );
   }
+  /*  add the faces */
+  for (i=0; i<set[setNr].anz_f; i++)
+  {
+    e=set[setNr].face[i];
+    // add the face as a possible target for projection purposes in copy_set(), they will not be copied 
+    seta( setcopy, "f", e );
+  }
   /*  add all elements */
   for (i=0; i<set[setNr].anz_e; i++)
   {
