@@ -2,8 +2,6 @@
 #ifdef WIN32
   #include <windows.h>
   #undef near
-  #undef max
-  #undef min
   #define DEV_NULL  " "
   #define DEV_NULL2 " "
 #else
@@ -12,6 +10,11 @@
   #include <unistd.h>
 #endif
 
+#ifdef DOUBLEPREC
+  #define CGXFLOAT double
+#else
+  #define CGXFLOAT float
+#endif
 #include <math.h>
 #include <string.h>
 #include <fcntl.h>
@@ -42,6 +45,7 @@ void printf_fflush(const char *fmt,...);
 #define     MAX_FLOAT   1.e32
 
 #define     MAX_LINE_LENGTH 256
+#define     OUTSIDE     1.e32
 #define     NODES       1000000
 #define     ELEMENTS    1000000
 
@@ -50,20 +54,25 @@ void printf_fflush(const char *fmt,...);
 #define     PNTS  10000
 
 /* from #include "f2c.h" */
+#undef max
+#undef min
 #define abs(x) ((x) >= 0 ? (x) : -(x))
 #define dabs(x) (double)abs(x)
-#define min(a,b) ((a) <= (b) ? (a) : (b))
-#define max(a,b) ((a) >= (b) ? (a) : (b))
-#define smin(a,b) (float)min(a,b)
-#define smax(a,b) (float)max(a,b)
-#define dmin(a,b) (double)min(a,b)
-#define dmax(a,b) (double)max(a,b)
+#define imin(a,b) ((a) <= (b) ? (a) : (b))
+#define imax(a,b) ((a) >= (b) ? (a) : (b))
+#define smin(a,b) (float)imin(a,b)
+#define smax(a,b) (float)imax(a,b)
+#define dmin(a,b) (double)imin(a,b)
+#define dmax(a,b) (double)imax(a,b)
 
 
 typedef struct {
   char  model[MAX_LINE_LENGTH]; /* model-name header*/
+  char  free;   /* 1: free data of current dataset if a new one is selected   */
+  char  threads;   /* nr of threads to be used */
   char  **uheader; /* user header */
   char  **pheader; /* project header (remark: leading dataset-project-headers are stored in the related dataset!) */
+  FILE  *filehandle; /* pointer to the frdfile, needed to close the file from outside */
   int   v;         /* number of values */
   int   u;         /* number of user headers */
   int   p;         /* number of project headers */
@@ -120,6 +129,7 @@ typedef struct {
                          /*  6: axisymmetric  (CAX) tr3c */
                          /*  7: fluid he8f */
                          /*  8: tet10m */
+                         /*  9: tet10t */
                          /*  14: reduced integration, plane strain (CPE)  */
                          /*  15: reduced integration, plane stress (CPS)  */
                          /*  16: reduced integration, axisymmetric  (CAX) */
@@ -150,12 +160,16 @@ typedef struct {
   int node_nr;           /* reference to node number  */ 
   int nFlag;             /* 0: no node-nr */
   int vFlag;             /* 0: no value */
+  int tFlag;             /* 0: no text */
   int fFlag;             /* 0: e-format, 1: float, 2: int */
+  int pFlag;             /* textpos 0: last, 1: bottom, 2: front 3: top */
   double tx;             /*   window coordinates */
   double ty;
   double mx;             /*   model coordinates */
   double my;
   double mz;
+  char   *text;          /* node related text */
+  char   *string;        /* complete string   */
 } Texts;
 
 
@@ -165,17 +179,27 @@ typedef struct {
   int i,j,k;              /* block dimension in i,j,k direction */
   int *nod;
   int geo;                /* related surf/body */
-  int bcface[6];          /* index of the line/face of the related surf/body */
-  int neighbor[6];        /* adjacent surface/body index */
+  int bcface[6];          /* edge index (line/face) of the related surf/body */
+  int neighbor[6];        /* adjacent surface/body index related to v:v<surf[j].nl */
   int map[6][3];          /* relative orientation of the neighbor 1:i==i_neigh, 4:i==-i_neigh */
   int strt1[6][3]; 
   int end_1[6][3]; 
   int strt2[6][3]; 
   int end_2[6][3];
-  char bctype[6][MAX_LINE_LENGTH];          /* bondary condition type */
+  char bctype[6][MAX_LINE_LENGTH];          /* boundary condition type */
 } NodeBlocks;
 
 
+typedef struct {
+  char name[MAX_LINE_LENGTH];          // setname with that boundary condition (in,out etc)
+  char bctype[MAX_LINE_LENGTH];        // subsonic-inflow etc, as given in send command
+  int surfs;                           // nr of surfaces (lines) in that set
+  int *surf;                           // duns surface-nr (line)
+  int *nBlock;                         // duns block-nr (cgx surf or body index +1)
+  int *side;                           // cgx surf or body edge index
+} NodeBlockbou;
+
+
 typedef struct {
   int nod[3];
   double ncol[3][3];
@@ -204,6 +228,8 @@ typedef struct {
   double *tcon, *con;
   int    nsph;                /* *SPECIFIC HEAT */
   double *tsph, *sph;
+  int    npl;                /* *PLASTIC */
+  double *spl, *epl, *tpl;
 } Materials;
 
 
@@ -217,13 +243,14 @@ typedef struct {
 typedef struct {
   char  **pheader;    /* project header */
   int   npheader;              /* number of headers */
+  int   numnod;              /* number of nodes in datablock */
   char  **compName;
   char  **icname;
   char  name[MAX_LINE_LENGTH];
   char  dataset_name[MAX_LINE_LENGTH];
   char  dataset_text[MAX_LINE_LENGTH];
   char  analysis_name[MAX_LINE_LENGTH];
-  float value;
+  double value;
   char  filename[MAX_LINE_LENGTH];
   FILE *handle;
   fpos_t *fileptr;
@@ -238,10 +265,17 @@ typedef struct {
   int *icind1;
   int *icind2;
   int *iexist;
+#ifdef DOUBLEPREC
+  double **dat;        /* node related data */
+  double ***edat;      /* element related data, not propper implemented */
+  double *max;         /* maximum datum */
+  double *min;         /* minimum datum */
+#else
   float **dat;        /* node related data */
   float ***edat;      /* element related data, not propper implemented */
   float *max;         /* maximum datum */
   float *min;         /* minimum datum */
+#endif
   int *nmax;          /* node with maximum datum */
   int *nmin;          /* node with minimum datum */
 } Datasets;
@@ -251,6 +285,7 @@ typedef struct {
   char *name;
   char flag;                  /* if the set is open: 'o' else: 'c' */
   char type;                  /* ordered entities:1 (seq) or not: 0 (set) */ 
+  char lock;                  /* if 1 seta and setr are locked for that set */ 
   int material;
   int index;           /* index of type-0 sets, assigned and updated in prnt(), eval. in getSetNr() */
   int anz_v;
@@ -362,13 +397,14 @@ typedef struct {
 typedef struct{
   char   *name;
   char   *entityName;
+  char   *type;
 } Alias;
 
 
 typedef struct {
   char  *name;
-  int type;              /* 0:plane, 1:cyl, 2: cone, 3:sph, 4:nurbs */
-  int p[7];              /* 0:3p, 1:3p, 2:4p 3:7p 4:p[0]=nurbsindx */
+  int type;              /* 0:plane, 1:cyl, 2: cone, 3:sph, 4:nurbs 5:torus */
+  int p[7];              /* 0:3p, 1:3p, 2:4p 3:7p 4:p[0]=nurbsindx 5:4p*/
   int ns;                /* nr of related surfs */
   int *s;                /* related surfs */
   GLint   npgn;          /* size of feed-back-Buffer pgn */
@@ -379,6 +415,7 @@ typedef struct {
 typedef struct {
   char  *name;
   char  *string;
+  char  flag;     // if 1 not to be written to fbd file
 } Values;
 
 
@@ -409,6 +446,7 @@ typedef struct {
   int   *elem;
   int  etyp;
   int  eattr;       /* -1:unstructured mesh, 0:default, 1:reduced integration, 2:incompatible modes */
+  int   elock;      /* 1: no element props can be changed */ 
 } Lines;
 
 typedef struct {
@@ -435,10 +473,12 @@ typedef struct {
   int   *cp;              /* index of points at the junction between  lines, lcmbs, starting at the beginning of the first line */
   int   nc;               /* number of closed-line-loops (curves) */
   int   *c;               /* number of lines in each curve */
+  int   sref;             /* if nl!=4: index of the 4 sided substitute surface, else -1 */
   double cx;               /* CG, in orient determined, not scaled during run! */
   double cy;
   double cz;
   int   fail;             /* if 1 then the meshing failed, if 2 trimming had failed */
+  int   permElem;         /* if 1 the mesher will generate elements, else only nodes */
   int   nn;
   int   *nod;
   int   ne;
@@ -446,6 +486,7 @@ typedef struct {
   int   etyp;
   int   eattr;       /* -1:unstructured mesh, 0:default, 1:reduced integration, 2:incompatible modes */
   char  *eparm;      /* parameter, so far used to store the mesh-density requirement for the mesher */
+  int   elock;      /* 1: no element props can be changed */ 
   int   patch;            /* number of trimming patch in the related nurbs */
   GLint   npgn;           /* size of feed-back-Buffer pgn */
   GLdouble *pgn;           /* stores poligons of the trimmed nurbs which defines the interiour */
@@ -453,6 +494,7 @@ typedef struct {
 
 typedef struct {
   char  *name;
+  char  typ;     /* 0: regular, 1:with unbalanced edges */
   char  ori;     /* body-orientation +- */
   int   ns;               /* nr. of surfaces (currently must be 6 for meshing) */
   char  *o;      /* s-orient +- */
@@ -460,7 +502,6 @@ typedef struct {
   double cx;               /* CG, in orient determined, not scaled during run! */
   double cy;
   double cz;
-  int   fail;             /* if 1 then the meshing failed */
   int   nn;
   int   *nod;
   int   ne;
@@ -468,6 +509,22 @@ typedef struct {
   int   etyp;
   int  eattr;       /* -1:unstructured mesh, 0:default, 1:reduced integration, 2:incompatible modes */
   char  *eparm;
+  int   elock;      /* 1: no element props can be changed */ 
+  // mesher data
+  int   fail;             /* if 1 then the meshing failed */
+  int   unbalance[7];     /* nr of unbalanced edges */
+  int div_l[7][5];
+  int umax;
+  int vmax;
+  int wmax;
+  int offs_sa1;
+  int offs_sa2;
+  int div_sa1;
+  int div_sa2;
+  int bs[6];
+  int ml[5];
+  char mlt[5];
+  int s_indx[6];
 } Gbod;
 
 typedef struct {
@@ -518,6 +575,10 @@ typedef struct {
 
   /* additional values for meshing purposes (needed for tr6u) */
   int nurbsType;            /* 0plate, 1cyl, 2torus, 3ball, 4half-ball-bot, 5half-ball-top */
+  int axis;
+  // axis=1 v is ambiguous (a given point could have vmax or vmin)
+  // axis=2 u is ambiguous (a given point could have umax or umin)
+  // axis=3 u and v are ambiguous (torus)
   int **sum_ambiguousPnts;  /* sum of double points in uv-space in each trimming curve */
   int **uvflipped;          /* 1 if the uv loop is reversed in a trimming curve*/
 } Nurbs;
@@ -526,16 +587,9 @@ typedef struct {
 
 typedef struct {
   double alpha, beta;
-  int nadapt;
+  int nadapt,tetmesher;
 } Meshp;
 
-typedef struct {
-  char name[MAX_LINE_LENGTH];
-  char bctype[MAX_LINE_LENGTH];
-  int surfs;
-  int *surf;
-} Dunsbou;
-
 
 typedef struct {
   int e;          /* mother element nr */
@@ -565,7 +619,7 @@ void define_rgb(float v, float *r, float *g, float *b);
 void text(double x, double y, double z,char *msg, void *glut_font);
 void scala_rgb( double dx, double dy, int divisions, double bmin, double bmax, double *col, void *glut_font, char format);
 void scala_indx( double dx, double dy, int divisions, double bmin, double bmax, int offset, int col, void *glut_font, char format);
-void scala_tex( double dx, double dy, int divisions, double bmin, double bmax, double scale, double *col, void *glut_font, char format, char fnr, char lnr);
+void scala_tex( double ratio, double dx, double dy, int divisions, double bmin, double bmax, double scale, double *col, void *glut_font, char format, char fnr, char lnr);
 int button(double dx, double dy, char *msg, double mx, double my, void *glut_font);
 void polymark ( int n, double *col_r, double *col_g, double *col_b, double *x,
 		double *y, double *z );
@@ -588,11 +642,12 @@ int getGeoDataTria( double *p1, double *p2, double *p3, double *Ix, double *Iy,
 int getrecord (int bufstart, int bufsize, char *inputdata, char *buffer);
 int gl3grades( long double a, long double b, long double c, double *x);
 int AsplitA( double *pa1, double *pa2, double *pa3, double *pb1, double *pb2, double *pb3, double *ps1, double *ps2);
-double AsplitL( double *b, double *eu, double *ev, double *eg );
+double AsplitL( double *b, double *eu, double *ev, double *eg, double *en );
 double interpol(double *x, double *y, int nn, double x0);
 double intpol2(double *x, double *y, int n, double x0, int *method );
 double intpol3(double *x, double *y, int n, double x0, int *method, double s, int iopt );
 double intpol(double *x, double *y, int n, double x0 );
+double normdist( double *p0, double *p1, double *p2, double *p3 );
 int nearNodes(double *po, int *n, double *p, int *node, int *k, int *flag);
 int iinsert(int **ipnt, int n, int x0 );
 int ifind(int **ipnt, int n, int x0 );
@@ -600,7 +655,6 @@ int iremove(int **ipnt, int n, int x0 );
 void linelength(double *x, double *y, double *z, int n, double *s );
 int kbrecord( char *string);
 int p3_finder(int anzahl_nodes, double *nx, double *nz, int *p);
-int parser( char gkey, char *record, int *curshft);
 void freeDataset(Datasets *lcase, int nr);
 int readAnsys(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr, double phi, int frequency );
 int readDyna( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
@@ -610,10 +664,11 @@ int readOneNode( int lc, Summen *anz, Datasets *lcase, int nodenr, double **vptr
 int read2frd(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
 int readFElt(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
 int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr, int elem_type );
-int readIsaac(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr, NodeBlocks **bptr, int elem_type,  double A, double B, double C );
+int readIsaac(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr, NodeBlocks **bptr, int elem_type);
 int readGiff( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
 int readNastran( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
 int readNG( char *datin, Summen *apre, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr );
+int readTG( char *datin, Summen *apre, Sets **sptr, Nodes **nptr, Elements **eptr, Datasets **lptr );
 int readStl( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr );
 int rotiere(double *x, double *y, int n, double phi, double *xneu, double *yneu);
 int checkIfNumber( char *string );
@@ -623,6 +678,7 @@ int stoi(char *string, int a, int b);
 void stos(char *string, int a, int b, char *puffer);
 int sins(char *string, int a, int b, char *puffer);
 int sword( char *string, char *word);
+int sword2( char *string, char *word);
 double p_angle(double x, double y);
 void v_add( double *A, double *B, double *C );
 double v_betrag(double *a);
@@ -639,6 +695,7 @@ double v_angle_ref( double *v0, double *v1, double *en );
 double v_distA( double *N0, double *N1, double *N2, double *N, double *vray, double  triScale, int *orient);
 int  v_rec2cyl( double *pr, int axis, int *csys, double *pc );
 int v_sgg( double *p1, double *p2, double *e1s, double *e2s, double *ps);
+void v_print( char *text, double *a);
 
 void vl_add( double *A, double *B, double *C );
 double vl_betrag(double *a);
@@ -658,13 +715,14 @@ int m_prodtr(int *n,double *a,double *b,double *c);
 void  m_sub(double *ms, double *m, double *s);
 int write2frd(char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, int binFlag );
 int write2nas(char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
-int write2aba(char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, char **dat );
+int write2aba(char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, char **dat, int controlFlag );
 int write2ansys(char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, char **dat );
 int write2aster(char *datout, Summen *anz, Nodes *node, Elements *elem, Sets *set, Datasets *lcase ); //TODD
 int write2darwin( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, char **dat );
+int write2dyna( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
 int write2samcef(char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase );
-int write2isaac( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, NodeBlocks *nBlock );
-int write2duns( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, NodeBlocks *nBlock, int bouNr, Dunsbou *dunsbou );
+int write2isaac( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, NodeBlocks *nBlock, int bouNr, NodeBlockbou *blockbou );
+int write2duns( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, NodeBlocks *nBlock, int bouNr, NodeBlockbou *blockbou );
 
 
 
