@@ -35,6 +35,8 @@ extern "C" {
 #endif
 
 #endif
+  #undef max
+  #undef min
 
 extern double     gtol;
 
@@ -43,6 +45,7 @@ extern char  printFlag;                   /* printf 1:on 0:off */
 extern Scale     scale[1];
 extern Summen    anz[1];
 extern Nodes     *node;
+extern Faces     *face;
 extern Elements  *e_enqire;
 
 extern SumGeo    anzGeo[1];
@@ -53,19 +56,12 @@ extern Sets      *setx;
 
 using namespace std;
 
-/*
-namespace nglib {
-#include <nglib.h>
-}
-*/
 
-int generateTetFromSet(int setNr, double teth )
+int generateTetFromSet(int setNr, double teth, int eattr, int mesherFlag )
 {
-  //using namespace nglib;
-
   int e,i,j,k,n;
   int snodSet, np, ne, ngtet[10], cgxtet[10], sumtri3=0, sumtri6=0,n1,n2,nm;
-  //double ngpnt[3];
+  int *ebuf;
   static int *cgxnode=NULL;
   static int *ngnode=NULL;
   char buffer[1000];
@@ -82,8 +78,6 @@ int generateTetFromSet(int setNr, double teth )
   Nodes     *node_ng=NULL;
   Elements  *elem_ng=NULL;
 
-  //Ng_Mesh * mesh;
-
   anz_ng->orign=0;
   anz_ng->n=0;
   anz_ng->e=0;
@@ -107,10 +101,6 @@ int generateTetFromSet(int setNr, double teth )
   // therefore a fresh start is needed
   setx=NULL;
 
-  //Ng_Meshing_Parameters mp;
-  //mp.maxh = teth/scale->w;
-  //mp.fineness = 1.;
-
   // check the consistency of the surface mesh. Either all tr6 or tr3
   // generate a list of surface nodes which will be passed to NG
 
@@ -139,61 +129,108 @@ int generateTetFromSet(int setNr, double teth )
     return(0);
   }
 
-  // creates mesh structure
-  //Ng_Init();
-  //mesh = Ng_NewMesh ();
-
   // add the nodes and elements to mesh
   if ((ngnode = (int *)realloc((int *)ngnode, (anz->nmax+1)*sizeof(int)) ) == NULL )
     { errMsg("ERROR: realloc failure in generateTet\n"); return(0); }
 
-  FILE *handle = fopen ("nodnr_ng_cgx.out", "w");
-  FILE *handle_ng = fopen ("mesh.ng", "w");
-
-  fprintf (handle_ng, "%d\n", set[snodSet].anz_n);
+  FILE *handle = fopen ("nodnr.out", "w");
+  FILE *handle_ng = NULL;
+  if(mesherFlag==0) {
+    handle_ng = fopen ("mesh.ng", "w");
+    fprintf (handle_ng, "%d\n", set[snodSet].anz_n); }
+  else {
+    handle_ng = fopen ("mesh.smesh", "w");
+    fprintf (handle_ng, "%d 3\n", set[snodSet].anz_n); }
+ 
   for (i = 0; i < set[snodSet].anz_n; i++)
   {
-    //Ng_AddPoint (mesh, &node[set[snodSet].node[i]].nx);                             
     ngnode[set[snodSet].node[i]]=i+1;
-    fprintf(handle,"ng %d cgx %d\n", i+1,set[snodSet].node[i]);
-    fprintf (handle_ng, "%.12e %.12e %.12e\n", node[set[snodSet].node[i]].nx, node[set[snodSet].node[i]].ny, node[set[snodSet].node[i]].nz);
+    if(mesherFlag==0) {
+      fprintf(handle,"ng %d cgx %d\n", i+1,set[snodSet].node[i]);
+      fprintf (handle_ng, "%.12e %.12e %.12e\n", node[set[snodSet].node[i]].nx, node[set[snodSet].node[i]].ny, node[set[snodSet].node[i]].nz); }
+    else {
+      fprintf(handle,"tg %d cgx %d\n", i+1,set[snodSet].node[i]);
+      fprintf (handle_ng, "%d %.12e %.12e %.12e\n", i+1,node[set[snodSet].node[i]].nx, node[set[snodSet].node[i]].ny, node[set[snodSet].node[i]].nz); }
   }
   fclose(handle);
 
-  handle = fopen ("elemnr_ng_cgx.out", "w");
+  handle = fopen ("elemnr.out", "w");
 
   fprintf (handle_ng, "%d\n", set[setNr].anz_e);
   for (i = 0; i < set[setNr].anz_e; i++)
   {
     for(k=0; k<3; k++) ngtet[k]=ngnode[e_enqire[set[setNr].elem[i]].nod[k]];
-    //Ng_AddSurfaceElement (mesh, NG_TRIG, ngtet);
     fprintf(handle,"ng %d cgx %d\n", i+1,set[setNr].elem[i]);
-    for(k=0; k<3; k++) fprintf (handle_ng, " %d",ngnode[e_enqire[set[setNr].elem[i]].nod[k]]); fprintf (handle_ng, "\n");
+    if(mesherFlag==1) fprintf (handle_ng, "3 ");
+    for(k=0; k<3; k++) fprintf (handle_ng, " %d",ngnode[e_enqire[set[setNr].elem[i]].nod[k]]);
+    fprintf (handle_ng, "\n");
   }
   fclose(handle);
   fclose(handle_ng);
 
-  // mesh
-  // either linked:
-  //Ng_GenerateVolumeMesh (mesh, &mp);
-  // or as stand alone mesher:
-  sprintf(buffer, "ng_vol mesh.ng %f%d", teth/scale->w, 0);
-  system(buffer);
-  printf (" Try to read NG file\n\n");
-  if ( (  readNG( "test.vol", anz_ng, &setx, &node_ng, &elem_ng, NULL)) == -1) 
+  // mesh, stand alone mesher:
+  if(mesherFlag==0)
   {
-    printf("ERROR: No mesh-file found\n\n");
-    return(0);
+    sprintf(buffer, "ng_vol mesh.ng %f%d", teth/scale->w, 0);
+    system(buffer);
+    printf (" Try to read NG file\n\n");
+    if ( (  readNG( "test.vol", anz_ng, &setx, &node_ng, &elem_ng, NULL)) == -1) 
+    {
+      printf("ERROR: No mesh-file found\n\n");
+      return(0);
+    }
   }
+  else
+  {
+    sprintf(buffer, "tetgen -qYO%da%e mesh.smesh",5, (teth/scale->w)*(teth/scale->w)*(teth/scale->w)/5.);
+    system(buffer);
+    printf (" Try to read TG file\n\n");
+    sprintf(buffer, "mesh.1.node");
+    if ( (  readTG( buffer, anz_ng, &setx, &node_ng, &elem_ng, NULL)) == -1) 
+    {
+      printf("ERROR: No mesh-file found\n\n");
+      return(0);
+    }
+  }
+
   for(j=0; j<anz_ng->sets; j++) {  delSetx(setx[j].name); }
   free(setx); setx=NULL;
   if(anz_ng->e>0)
   {
-    system("rm -rf mesh.ng");
-    system("rm -rf test.vol");
-    system("rm -rf test.out");
-    system("rm -rf netgen.prof");
-    system("rm -rf nodnr_ng_cgx.out elemnr_ng_cgx.out");
+    if(mesherFlag==0)
+    {
+#ifdef WIN32
+      system("del /f \"mesh.ng\"");
+      system("del /f \"test.vol\"");
+      system("del /f \"test.out\"");
+      system("del /f \"netgen.prof\"");
+      system("del /f \"nodnr.out\" \"elemnr.out\"");
+#else
+      system("rm -rf mesh.ng");
+      system("rm -rf test.vol");
+      system("rm -rf test.out");
+      system("rm -rf netgen.prof");
+      system("rm -rf nodnr.out elemnr.out");
+#endif
+    }
+    else
+    {
+#ifdef WIN32
+      system("del /f \"mesh.smesh\"");
+      system("del /f \"mesh.1.node\"");
+      system("del /f \"mesh.1.ele\"");
+      system("del /f \"mesh.1.face\"");
+      system("del /f \"mesh.1.edge\"");
+      system("del /f \"nodnr.out\" \"elemnr.out\"");
+#else
+      system("rm -rf mesh.smesh");
+      system("rm -rf mesh.1.node");
+      system("rm -rf mesh.1.ele");
+      system("rm -rf mesh.1.face");
+      system("rm -rf mesh.1.edge");
+      system("rm -rf nodnr.out elemnr.out");
+#endif
+    }
   }
   else
   {
@@ -202,7 +239,6 @@ int generateTetFromSet(int setNr, double teth )
   }
 
   // define new nodes and tets
-  //np = Ng_GetNP(mesh);
   np = anz_ng->n;
   if ((cgxnode = (int *)realloc((int *)cgxnode, (np+1)*sizeof(int)) ) == NULL )
     { errMsg("ERROR: realloc failure in generateTet\n"); return(0); }
@@ -213,8 +249,6 @@ int generateTetFromSet(int setNr, double teth )
     else
     {
       cgxnode[i+1]=anz->nnext++;
-      //Ng_GetPoint (mesh, i+1, ngpnt);
-      //nod( anz, &node, 1, cgxnode[i+1], ngpnt[0], ngpnt[1], ngpnt[2], 0 );     
       nod( anz, &node, 1, cgxnode[i+1], node_ng[i+1].nx, node_ng[i+1].ny, node_ng[i+1].nz, 0 );     
     } 
   }
@@ -262,8 +296,12 @@ int generateTetFromSet(int setNr, double teth )
     {
       for (j=0; j<set[setNr].anz_e; j++)
       {
-        if(surf[k].elem[n]==set[setNr].elem[j]) surf[k].elem[n]=0;
+        if(surf[k].elem[n]==set[setNr].elem[j])
+	{
+          face[ face[surf[k].elem[n]].indx[1] ].nr=-1;
+          surf[k].elem[n]=0;
   	}
+      }
     }
     e=0;
     for(n=0; n<surf[k].ne; n++) if(surf[k].elem[n]>0) surf[k].elem[e++]=surf[k].elem[n];
@@ -274,17 +312,26 @@ int generateTetFromSet(int setNr, double teth )
   {
     if(( set[k].name != (char *)NULL )&&( k != setNr))
     {
-      for (j=0; j<set[setNr].anz_e; j++) setr( k, "e", set[setNr].elem[j]);
+      if(set[k].anz_e<=0) continue;
+      if( (ebuf=(int *)calloc((anz->emax+1), sizeof(int) ) )==NULL) 
+	{ printf(" ERROR: calloc failure\n"); return(0); }
+      for(j=0; j<set[k].anz_e; j++) ebuf[set[k].elem[j]]=1;
+      if(set[k].type==0)
+      {
+        n=set[k].elem[set[k].anz_e-1]; // max elemnr in sorted set
+        for(j=0; j<set[setNr].anz_e; j++) { if(set[setNr].elem[j]<=n) ebuf[set[setNr].elem[j]]=0; else break; }
+      }
+      else { for(j=0; j<set[setNr].anz_e; j++) ebuf[set[setNr].elem[j]]=0; } 
+      e=0;
+      for(j=0; j<set[k].anz_e; j++) if(ebuf[set[k].elem[j]]>0) set[k].elem[e++]=set[k].elem[j];
+      set[k].anz_e=e;
+      free(ebuf);
     }
   }
 
-  //ne = Ng_GetNE(mesh);
   ne = anz_ng->e;
-  //for (i = 0; i < ne; i++)
   i=0; for (j = 0; j < ne; j++)
   {
-    //if(Ng_GetVolumeElement (mesh, i+1, ngtet)!=NG_TET) printf("ERROR: e:%d not tet\n",i+1);
-    //printf("e:%d type:tet \n",i+1);
     if(elem_ng[j].type != 3) continue;
     ngtet[0]=elem_ng[j].nod[0];
     ngtet[2]=elem_ng[j].nod[1];
@@ -296,8 +343,8 @@ int generateTetFromSet(int setNr, double teth )
     cgxtet[1]=cgxnode[ngtet[2]];
     cgxtet[2]=cgxnode[ngtet[1]];
     cgxtet[3]=cgxnode[ngtet[3]];
-    if(i<set[setNr].anz_e) elem_define( set[setNr].elem[i], 3, cgxtet, 1, 0 );
-    else { elem_define( anz->enext++, 3, cgxtet, 1, 0 ); seta( setNr, "e", anz->emax ); }
+    if(i<set[setNr].anz_e) elem_define(anz,&e_enqire, set[setNr].elem[i], 3, cgxtet, 1, eattr );
+    else { elem_define(anz,&e_enqire, anz->enext++, 3, cgxtet, 1, eattr ); seta( setNr, "e", anz->emax ); }
     i++;
   }
   ne = i;
@@ -340,20 +387,16 @@ int generateTetFromSet(int setNr, double teth )
     fixMidsideNodes( set[setNr].name, "" );
   }
 
-  //Ng_Exit();
   printf("tet-mesh done with h:%f\n",teth);
   return(ne);
 }
 
 
 
-int generateTetFromBody(int nr, double teth )
+int generateTetFromBody(int nr, double teth, int eattr, int mesherFlag)
 {
-  //using namespace nglib;
-
   int i,j,k,n,s,sb;
   int setNr, snodSet, snodSet2, mnodSet, np, ne, ngtet[10], cgxtet[10], sumtri=0, sumtri3=0, sumtri6=0,n1,n2,nm;
-  //double ngpnt[3];
   static int *cgxnode=NULL;
   static int *ngnode=NULL;
   char buffer[1000];
@@ -371,8 +414,6 @@ int generateTetFromBody(int nr, double teth )
   Nodes     *node_ng=NULL;
   Elements  *elem_ng=NULL;
 
-  //Ng_Mesh * mesh;
-
   anz_ng->orign=0;
   anz_ng->n=0;
   anz_ng->e=0;
@@ -396,14 +437,9 @@ int generateTetFromBody(int nr, double teth )
   // therefore a fresh start is needed
   setx=NULL;
 
-  //Ng_Meshing_Parameters mp;
-  //mp.maxh = teth/scale->w;
-  //mp.fineness = 1.;
-
   // check the consistency of the surface mesh. Either all tr6 or tr3
   // generate a list of surface nodes which will be passed to NG
 
-  
   delSet("+snodSet");
   delSet("+mnodSet");
   delSet("+velemSet");
@@ -441,29 +477,33 @@ int generateTetFromBody(int nr, double teth )
     return(0);
   }
 
-  // creates mesh structure
-  //Ng_Init();
-  //mesh = Ng_NewMesh ();
-
   // add the nodes and elements to mesh
   if ((ngnode = (int *)realloc((int *)ngnode, (anz->nmax+1)*sizeof(int)) ) == NULL )
     { errMsg("ERROR: realloc failure in generateTet\n"); return(0); }
 
  tryToFlipBodyMark:;
-  FILE *handle = fopen ("nodnr_ng_cgx.out", "w");
-  FILE *handle_ng = fopen ("mesh.ng", "w");
-
-  fprintf (handle_ng, "%d\n", set[snodSet].anz_n);
+  FILE *handle = fopen ("nodnr.out", "w");
+  FILE *handle_ng = NULL;
+  if(mesherFlag==0) {
+    handle_ng = fopen ("mesh.ng", "w");
+    fprintf (handle_ng, "%d\n", set[snodSet].anz_n); }
+  else {
+    handle_ng = fopen ("mesh.smesh", "w");
+    fprintf (handle_ng, "%d 3\n", set[snodSet].anz_n); }
+ 
   for (i = 0; i < set[snodSet].anz_n; i++)
   {
-    //Ng_AddPoint (mesh, &node[set[snodSet].node[i]].nx);                             
     ngnode[set[snodSet].node[i]]=i+1;
-    fprintf(handle,"ng %d cgx %d\n", i+1,set[snodSet].node[i]);
-    fprintf (handle_ng, "%.12e %.12e %.12e\n", node[set[snodSet].node[i]].nx, node[set[snodSet].node[i]].ny, node[set[snodSet].node[i]].nz);
+    if(mesherFlag==0) {
+      fprintf(handle,"ng %d cgx %d\n", i+1,set[snodSet].node[i]);
+      fprintf (handle_ng, "%.12e %.12e %.12e\n", node[set[snodSet].node[i]].nx, node[set[snodSet].node[i]].ny, node[set[snodSet].node[i]].nz); }
+    else {
+      fprintf(handle,"tg %d cgx %d\n", i+1,set[snodSet].node[i]);
+      fprintf (handle_ng, "%d %.12e %.12e %.12e\n", i+1,node[set[snodSet].node[i]].nx, node[set[snodSet].node[i]].ny, node[set[snodSet].node[i]].nz); }
   }
   fclose(handle);
 
-  handle = fopen ("elemnr_ng_cgx.out", "w");
+  handle = fopen ("elemnr.out", "w");
 
   //printf("body:%s\n", body[nr].name);
   n=0; for(sb=0; sb<body[nr].ns; sb++) n+=surf[body[nr].s[sb]].ne;
@@ -481,9 +521,10 @@ int generateTetFromBody(int nr, double teth )
       for (i = 0; i < surf[s].ne; i++)
       {
         j=2; for(k=0; k<3; k++) ngtet[k]=ngnode[e_enqire[surf[s].elem[i]].nod[j--]];
-        //Ng_AddSurfaceElement (mesh, NG_TRIG, ngtet);
         fprintf(handle,"ng %d cgx %d\n", i+1,surf[s].elem[i] );
-        for(k=0; k<3; k++) fprintf (handle_ng, " %d",ngtet[k]); fprintf (handle_ng, "\n");
+        if(mesherFlag==1) fprintf (handle_ng, "3 ");
+        for(k=0; k<3; k++) fprintf (handle_ng, " %d",ngtet[k]);
+        fprintf (handle_ng, "\n");
       }
     }
     else
@@ -492,40 +533,87 @@ int generateTetFromBody(int nr, double teth )
       for (i = 0; i < surf[s].ne; i++)
       {
         for(k=0; k<3; k++) ngtet[k]=ngnode[e_enqire[surf[s].elem[i]].nod[k]];
-        //Ng_AddSurfaceElement (mesh, NG_TRIG, ngtet);
         fprintf(handle,"ng %d cgx %d\n", i+1,surf[s].elem[i] );
-        for(k=0; k<3; k++) fprintf (handle_ng, " %d",ngtet[k]); fprintf (handle_ng, "\n");
+        if(mesherFlag==1) fprintf (handle_ng, "3 ");
+        for(k=0; k<3; k++) fprintf (handle_ng, " %d",ngtet[k]);
+        fprintf (handle_ng, "\n");
       }
     }
   }
   fclose(handle);
   fclose(handle_ng);
 
-  // mesh
-  // either linked:
-  //Ng_GenerateVolumeMesh (mesh, &mp);
-  // or as stand alone mesher:
-  sprintf(buffer, "ng_vol mesh.ng %f%d", teth/scale->w, 0);
-  system(buffer);
-  printf (" Try to read NG file\n\n");
-  if ( (  readNG( "test.vol", anz_ng, &setx, &node_ng, &elem_ng, NULL)) == -1) 
+  // mesh, stand alone mesher:
+  if(mesherFlag==0)
   {
-    if(tryToFlipBody)
+    sprintf(buffer, "ng_vol mesh.ng %f%d", teth/scale->w, 0);
+    system(buffer);
+    printf (" Try to read NG file\n\n");
+    if ( (  readNG( "test.vol", anz_ng, &setx, &node_ng, &elem_ng, NULL)) == -1) 
     {
-      printf("ERROR: No mesh-file found\n\n");
-      return(0);
+      if(tryToFlipBody)
+      {
+        printf("ERROR: No mesh-file found\n\n");
+        return(0);
+      }
+      else { tryToFlipBody=1; goto tryToFlipBodyMark; }
     }
-    else { tryToFlipBody=1; goto tryToFlipBodyMark; }
   }
+  else
+  {
+    sprintf(buffer, "tetgen -qY mesh.smesh");
+    system(buffer);
+    printf (" Try to read TG file\n\n");
+    sprintf(buffer, "mesh.1.node");
+    if ( (  readTG( buffer, anz_ng, &setx, &node_ng, &elem_ng, NULL)) == -1) 
+    {
+      if(tryToFlipBody)
+      {
+        printf("ERROR: No mesh-file found\n\n");
+        return(0);
+      }
+      else { tryToFlipBody=1; goto tryToFlipBodyMark; }
+    }
+  }
+
   for(j=0; j<anz_ng->sets; j++) {  delSetx(setx[j].name); }
   free(setx); setx=NULL;
   if(anz_ng->e>0)
   {
-    system("rm -rf mesh.ng");
-    system("rm -rf test.vol");
-    system("rm -rf test.out");
-    system("rm -rf netgen.prof");
-    system("rm -rf nodnr_ng_cgx.out elemnr_ng_cgx.out");
+    if(mesherFlag==0)
+    {
+#ifdef WIN32
+      system("del /f \"mesh.ng\"");
+      system("del /f \"test.vol\"");
+      system("del /f \"test.out\"");
+      system("del /f \"netgen.prof\"");
+      system("del /f \"nodnr.out\" \"elemnr.out\"");
+#else
+      system("rm -rf mesh.ng");
+      system("rm -rf test.vol");
+      system("rm -rf test.out");
+      system("rm -rf netgen.prof");
+      system("rm -rf nodnr.out elemnr.out");
+#endif
+    }
+    else
+    {
+#ifdef WIN32
+      system("del /f \"mesh.smesh\"");
+      system("del /f \"mesh.1.node\"");
+      system("del /f \"mesh.1.ele\"");
+      system("del /f \"mesh.1.face\"");
+      system("del /f \"mesh.1.edge\"");
+      system("del /f \"nodnr.out\" \"elemnr.out\"");
+#else
+      system("rm -rf mesh.smesh");
+      system("rm -rf mesh.1.node");
+      system("rm -rf mesh.1.ele");
+      system("rm -rf mesh.1.face");
+      system("rm -rf mesh.1.edge");
+      system("rm -rf nodnr.out elemnr.out");
+#endif
+    }
   }
   else
   {
@@ -534,7 +622,6 @@ int generateTetFromBody(int nr, double teth )
   }
 
   // define new nodes and tets
-  //np = Ng_GetNP(mesh);
   np = anz_ng->n;
   if ((cgxnode = (int *)realloc((int *)cgxnode, (np+1)*sizeof(int)) ) == NULL )
     { errMsg("ERROR: realloc failure in generateTet\n"); return(0); }
@@ -545,8 +632,6 @@ int generateTetFromBody(int nr, double teth )
     else
     {
       cgxnode[i+1]=anz->nnext++;
-      //Ng_GetPoint (mesh, i+1, ngpnt);
-      //nod( anz, &node, 1, cgxnode[i+1], ngpnt[0], ngpnt[1], ngpnt[2], 0 );     
       nod( anz, &node, 1, cgxnode[i+1], node_ng[i+1].nx, node_ng[i+1].ny, node_ng[i+1].nz, 0 );     
     } 
   }
@@ -590,7 +675,6 @@ int generateTetFromBody(int nr, double teth )
     }
   }
 
-  //ne = Ng_GetNE(mesh);
   ne = anz_ng->e;
 
   /* allocate memory for embeded elements */
@@ -599,11 +683,8 @@ int generateTetFromBody(int nr, double teth )
   if((body[nr].nod=(int *)realloc((int *)body[nr].nod, (ne)*sizeof(int)) )==NULL)
   { printf(" ERROR: realloc failure in generateTet body:%s can not be meshed\n\n" , body[nr].name); return(0); }
 
-  //for (i = 0; i < ne; i++)
   i=0; for (j = 0; j < ne; j++)
   {
-    //if(Ng_GetVolumeElement (mesh, i+1, ngtet)!=NG_TET) printf("ERROR: e:%d not tet\n",i+1);
-    //printf("e:%d type:tet \n",i+1);
     if(elem_ng[j].type != 3) continue;
     ngtet[0]=elem_ng[j].nod[0];
     ngtet[2]=elem_ng[j].nod[1];
@@ -616,7 +697,7 @@ int generateTetFromBody(int nr, double teth )
     cgxtet[2]=cgxnode[ngtet[1]];
     cgxtet[3]=cgxnode[ngtet[3]];
     body[nr].elem[i]=anz->enext;
-    elem_define( anz->enext++, 3, cgxtet, 1, 0 ); seta( setNr, "e", anz->emax );
+    elem_define(anz,&e_enqire, anz->enext++, 3, cgxtet, 1, eattr ); seta( setNr, "e", anz->emax );
     i++;
   }
   ne = i;
@@ -672,7 +753,7 @@ int generateTetFromBody(int nr, double teth )
   for (k = 0; k < set[mnodSet].anz_n; k++) setr(setNr,"n",set[mnodSet].node[k]);
 
   /* add to body */
-  if((body[nr].nod=(int *)realloc((int *)body[nr].nod, (set[setNr].anz_n)*sizeof(int)) )==NULL)
+  if((body[nr].nod=(int *)realloc((int *)body[nr].nod, (set[setNr].anz_n+1)*sizeof(int)) )==NULL)
   { printf(" ERROR: realloc failure in generateTet body:%s can not be meshed\n\n" , body[nr].name); return(0); }
   for (k = 0; k < set[setNr].anz_n; k++) body[nr].nod[k]=set[setNr].node[k];
   body[nr].nn=set[setNr].anz_n;
@@ -681,8 +762,6 @@ int generateTetFromBody(int nr, double teth )
   delSet("+mnodSet");
   delSet("+velemSet");
 
-
-  //Ng_Exit();
   printf("tet-mesh done with h:%f\n",teth);
   return(ne);
 }
