@@ -51,7 +51,7 @@ extern int       elemMat[MAX_MATERIALS];      /*  Material Numbers, Number of Ma
 /* for the CFD reader (ISAAC) */
 extern double pref, tref, R_GAS;
 
-int setall;
+extern int setall;
 
 
 void vdsort ( int *a ) {
@@ -990,7 +990,7 @@ void makeSurfaces()
       }
       for(j=0; j<set[i].anz_f; j++)
       {
-        if( face[set[i].face[j]].nr>-1)
+        if((face[set[i].face[j]].nr>-1)&&(face[set[i].face[j]].elem_nr<=anz->emax)&&(e_enqire[face[set[i].face[j]].elem_nr].type>0))
         {
           n=seta(i,"j",0);
           if(n>-1)
@@ -1077,7 +1077,6 @@ void makeSurfaces()
   }
   free(nclo);
   nclo=NULL;
-
   getFaceNormalen( face, node, anz );
 }
 
@@ -1094,6 +1093,11 @@ void iniMeshData( char *datin, char *format )
     printf (" Try to read ansysList file\n\n");
     if ( (  readAnsysList( datin, anz, &set, &node, &elem, &lcase)) == -1) { exit(-1);}
   }
+  else if(compare(format,"dynl",4)==4)
+  { 
+    printf (" Try to read dyna3d-list results\n\n");
+    if ( (  readDynaList( datin, anz, &set, &node, &elem, &lcase)) == -1) { exit(-1);}
+  }
   else if(compare(format,"frd",3)==3)
   { 
     printf (" Try to read ccx results\n\n");
@@ -1102,23 +1106,33 @@ void iniMeshData( char *datin, char *format )
   else if(compare(format,"ccx",3)==3)
   { 
     printf (" Try to read ccx input\n\n");
-    if ( (readccx( datin, anz, &set, &node, &elem, &lcase)) == -1) { exit(-1);}
+    if ( (readccx( datin, anz, &set, &node, &elem, &lcase, 1)) == -1) { exit(-1);}
   }
   else if(compare(format,"isaac2d",7)==7)
   { 
     printf (" Try to read 2D-isaac results\n\n");
-    if ( (  readIsaac( datin, anz, &node, &elem, &lcase, &nBlock, 9, pref, tref, R_GAS)) == -1) { exit(-1);}
+    if ( (  readIsaac( datin, anz, &node, &elem, &lcase, &nBlock, 9)) == -1) { exit(-1);}
   }
   else if(compare(format,"isaac3d",7)==7)
   { 
     printf (" Try to read 3D-isaac results\n\n");
-    if ( (  readIsaac( datin, anz, &node, &elem, &lcase, &nBlock, 1, pref, tref, R_GAS)) == -1) { exit(-1);}
+    if ( (  readIsaac( datin, anz, &node, &elem, &lcase, &nBlock, 1)) == -1) { exit(-1);}
+  }
+  else if(compare(format,"duns2dl",7)==7)
+  { 
+    printf (" Try to read 2D-duns results\n\n");
+    if ( (  readDuns( datin, anz, &node, &elem, &lcase, -9)) == -1) { exit(-1);}
   }
   else if(compare(format,"duns2d",6)==6)
   { 
     printf (" Try to read 2D-duns results\n\n");
     if ( (  readDuns( datin, anz, &node, &elem, &lcase, 9)) == -1) { exit(-1);}
   }
+  else if(compare(format,"duns3dl",7)==7)
+  { 
+    printf (" Try to read 3D-duns results\n\n");
+    if ( (  readDuns( datin, anz, &node, &elem, &lcase, -1)) == -1) { exit(-1);}
+  }
   else if(compare(format,"duns3d",6)==6)
   { 
     printf (" Try to read 3D-duns results\n\n");
@@ -1129,6 +1143,11 @@ void iniMeshData( char *datin, char *format )
     printf (" Try to read foam results\n\n");
     if ( (  readFoam( datin, anz, &set, &node, &elem, &lcase)) == -1) { exit(-1);}
   }
+  else if(compare(format,"vtk",3)==3)
+  { 
+    printf (" Try to read vtk file\n\n");
+    if ( (  readVtk( datin, anz, &node, &elem, &lcase)) == -1) { exit(-1);}
+  }
   else if(compare(format,"nas",2)==2)
   { 
     printf (" Try to read nastran f06 file\n\n");
@@ -1139,6 +1158,11 @@ void iniMeshData( char *datin, char *format )
     printf (" Try to read NG file\n\n");
     if ( (  readNG( datin, anz, &set, &node, &elem, &lcase)) == -1) { exit(-1);}
   }
+  else if(compare(format,"tg",2)==2)
+  { 
+    printf (" Try to read Tetgen file\n\n");
+    if ( (  readTG( datin, anz, &set, &node, &elem, &lcase)) == -1) { exit(-1);}
+  }
   else if(compare(format,"stl",3)==3)
   { 
     printf (" Try to read stl file\n\n");
@@ -1165,6 +1189,7 @@ void iniMeshData( char *datin, char *format )
   /* initialize the normal vector on each node */
   for (i=0; i<anz->n; i++) for(j=0; j<3; j++) node[node[i].nr].nv[j]=0.;
   iniElements(anz, elem, 0);
+  //printf("scal %f %f %f %f\n",scale->w,scale->x,scale->y,scale->z);
   if(anz->n>0)
   {
     if ( (set[setall].node = (int *)realloc((int *)set[setall].node, (anz->n+1) * sizeof(int))) == NULL )
@@ -1174,6 +1199,7 @@ void iniMeshData( char *datin, char *format )
     {
       set[setall].node[i]= node[i].nr;
       set[setall].anz_n++;
+      //printf(" node:%d %f %f %f\n",node[i].nr,node[node[i].nr].nx,node[node[i].nr].ny,node[node[i].nr].nz);
     }
     qsort( set[setall].node, set[setall].anz_n, sizeof(int), (void *)compareInt );
   }
@@ -1265,7 +1291,7 @@ void iniMeshData( char *datin, char *format )
 
 void iniElements(Summen *anz, Elements *elem, int ini_e_enqire)
 {
-  int  i, k, n, n1, n2;
+  int  i, k, n;
   int ipuf, nf=0;
 
   anz->enext   = anz->emax+1;
@@ -1279,31 +1305,6 @@ void iniElements(Summen *anz, Elements *elem, int ini_e_enqire)
 
   /* ------------- fill the element struktures -------------------------------- */
 
-  /* free all faces */
-  for(i=0; i<anz->f; i++)
-  {
-    if (face[i].type == 7)       nf=1;  /* TRI3  */
-    else if (face[i].type == 8)  nf=4; /* TRI6  */
-    else if (face[i].type == 9)  nf=2; /* QUAD4 */
-    else if (face[i].type == 10) nf=8; /* QUAD8 */
-    else if (face[i].type == 11) nf=1; /* BEAM */
-    else if (face[i].type == 12) nf=2; /* BEAM3 */
-    if(face[i].side!=NULL)
-    {
-      for(k=0; k<nf; k++) free(face[i].side[k]);
-      free(face[i].side);
-      face[i].side=NULL;
-    }
-  }
-
-  /* delete all faces from all sets */
-  for (i=0; i<anz->sets; i++ )
-  {
-    set[i].anz_f = 0;
-    free(set[i].face);
-    set[i].face=NULL;
-  }
-
   anz->g=0;
   anz->f=0;
 
@@ -1338,6 +1339,7 @@ void iniElements(Summen *anz, Elements *elem, int ini_e_enqire)
   /* e_enqire is used as e_enqire[elem[i].nr].xx   */
   if ( (e_enqire = (Elements *)realloc((Elements *)e_enqire, (anz->emax+1) * sizeof(Elements))) == NULL )
     printf(" ERROR: realloc in iniMeshData(), e_enqire\n\n") ;
+  for (i=0; i<anz->emax; i++) e_enqire[i].type=0;
   for (i=0; i<anz->e; i++)
   {
     e_enqire[i].nr    = elem[i].nr;
@@ -1422,241 +1424,10 @@ void iniElements(Summen *anz, Elements *elem, int ini_e_enqire)
     qsort( set[setall].elem, set[setall].anz_e, sizeof(int), (void *)compareInt );
   }
 
-  /* midside nodes must be generated afer element correctrion */
-  elemChecker( set[setall].anz_e, set[setall].elem, node, e_enqire);
-  for (k=0; k<anz->e; k++)
-  {
-    i=e_enqire[k].nr;
-    if (e_enqire[i].type == 4)
-    {
-      if ( (node = (Nodes *)realloc( (Nodes *)node, (anz->nmax+7) * sizeof(Nodes))) == NULL )
-        printf("\n\n ERROR: realloc failed node\n\n") ;
-      for (n=0; n<3; n++)  
-      {
-        anz->nmax++;
-        node[anz->n].nr = anz->nmax;
-        node[anz->nmax].pflag=1;
-        node[anz->nmax].indx=anz->n;
+  /* midside nodes must be generated after element correction */
+  //elemChecker( set[setall].anz_e, set[setall].elem, node, e_enqire);
+  adjustDrawNodes(1);
 
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[0+n]].nx+node[e_enqire[i].nod[1+n]].nx    +
-          node[e_enqire[i].nod[5+n]].nx+node[e_enqire[i].nod[4+n]].nx )  + 1./2.*(
-          node[e_enqire[i].nod[8+n]].nx+node[e_enqire[i].nod[13+n]].nx   +
-          node[e_enqire[i].nod[16+n]].nx+node[e_enqire[i].nod[12+n]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[0+n]].ny+node[e_enqire[i].nod[1+n]].ny    +
-          node[e_enqire[i].nod[5+n]].ny+node[e_enqire[i].nod[4+n]].ny )  + 1./2.*(
-          node[e_enqire[i].nod[8+n]].ny+node[e_enqire[i].nod[13+n]].ny   +
-          node[e_enqire[i].nod[16+n]].ny+node[e_enqire[i].nod[12+n]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[0+n]].nz+node[e_enqire[i].nod[1+n]].nz    +
-          node[e_enqire[i].nod[5+n]].nz+node[e_enqire[i].nod[4+n]].nz )  + 1./2.*(
-          node[e_enqire[i].nod[8+n]].nz+node[e_enqire[i].nod[13+n]].nz   +
-          node[e_enqire[i].nod[16+n]].nz+node[e_enqire[i].nod[12+n]].nz) ;
-
-        e_enqire[i].nod[n+20]=node[anz->n].nr;
-        anz->n++;
-      }
-        anz->nmax++;
-        node[anz->n].nr = anz->nmax;
-        node[anz->nmax].pflag=1;
-        node[anz->nmax].indx=anz->n;
-
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[3]].nx+node[e_enqire[i].nod[0]].nx    +
-          node[e_enqire[i].nod[4]].nx+node[e_enqire[i].nod[7]].nx )  + 1./2.*(
-          node[e_enqire[i].nod[11]].nx+node[e_enqire[i].nod[12]].nx   +
-          node[e_enqire[i].nod[19]].nx+node[e_enqire[i].nod[15]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[3]].ny+node[e_enqire[i].nod[0]].ny    +
-          node[e_enqire[i].nod[4]].ny+node[e_enqire[i].nod[7]].ny )  + 1./2.*(
-          node[e_enqire[i].nod[11]].ny+node[e_enqire[i].nod[12]].ny   +
-          node[e_enqire[i].nod[19]].ny+node[e_enqire[i].nod[15]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[3]].nz+node[e_enqire[i].nod[0]].nz    +
-          node[e_enqire[i].nod[4]].nz+node[e_enqire[i].nod[7]].nz )  + 1./2.*(
-          node[e_enqire[i].nod[11]].nz+node[e_enqire[i].nod[12]].nz   +
-          node[e_enqire[i].nod[19]].nz+node[e_enqire[i].nod[15]].nz) ;
-
-        e_enqire[i].nod[23]=node[anz->n].nr;
-        anz->n++;
-
-      for (n=0; n<2; n++) 
-      {
-        anz->nmax++;
-        node[anz->n].nr = anz->nmax;
-        node[anz->nmax].pflag=1;
-        node[anz->nmax].indx=anz->n;
-
-        n1=n*4;
-        n2=n*8;
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[0+n1]].nx+node[e_enqire[i].nod[1+n1]].nx    +
-          node[e_enqire[i].nod[2+n1]].nx+node[e_enqire[i].nod[3+n1]].nx )  + 1./2.*(
-          node[e_enqire[i].nod[8+n2]].nx+node[e_enqire[i].nod[9+n2]].nx   +
-          node[e_enqire[i].nod[10+n2]].nx+node[e_enqire[i].nod[11+n2]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[0+n1]].ny+node[e_enqire[i].nod[1+n1]].ny    +
-          node[e_enqire[i].nod[2+n1]].ny+node[e_enqire[i].nod[3+n1]].ny )  + 1./2.*(
-          node[e_enqire[i].nod[8+n2]].ny+node[e_enqire[i].nod[9+n2]].ny   +
-          node[e_enqire[i].nod[10+n2]].ny+node[e_enqire[i].nod[11+n2]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[0+n1]].nz+node[e_enqire[i].nod[1+n1]].nz    +
-          node[e_enqire[i].nod[2+n1]].nz+node[e_enqire[i].nod[3+n1]].nz )  + 1./2.*(
-          node[e_enqire[i].nod[8+n2]].nz+node[e_enqire[i].nod[9+n2]].nz   +
-          node[e_enqire[i].nod[10+n2]].nz+node[e_enqire[i].nod[11+n2]].nz) ;
-
-        e_enqire[i].nod[n+24]=node[anz->n].nr;
-        anz->n++;
-      }
-    }
-    if (e_enqire[i].type == 5)
-    {
-      if ( (node = (Nodes *)realloc( (Nodes *)node, (anz->nmax+6) * sizeof(Nodes))) == NULL )
-        printf("\n\n ERROR: realloc failed node\n\n") ;
-      for (n=0; n<2; n++)  
-      {
-        anz->nmax++;
-        node[anz->n].nr = anz->nmax;
-        node[anz->nmax].pflag=1;
-        node[anz->nmax].indx=anz->n;
-
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[0+n]].nx+node[e_enqire[i].nod[1+n]].nx    +
-          node[e_enqire[i].nod[4+n]].nx+node[e_enqire[i].nod[3+n]].nx )  + 1./2.*(
-          node[e_enqire[i].nod[6+n]].nx+node[e_enqire[i].nod[10+n]].nx   +
-          node[e_enqire[i].nod[12+n]].nx+node[e_enqire[i].nod[ 9+n]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[0+n]].ny+node[e_enqire[i].nod[1+n]].ny    +
-          node[e_enqire[i].nod[4+n]].ny+node[e_enqire[i].nod[3+n]].ny )  + 1./2.*(
-          node[e_enqire[i].nod[6+n]].ny+node[e_enqire[i].nod[10+n]].ny   +
-          node[e_enqire[i].nod[12+n]].ny+node[e_enqire[i].nod[ 9+n]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[0+n]].nz+node[e_enqire[i].nod[1+n]].nz    +
-          node[e_enqire[i].nod[4+n]].nz+node[e_enqire[i].nod[3+n]].nz )  + 1./2.*(
-          node[e_enqire[i].nod[6+n]].nz+node[e_enqire[i].nod[10+n]].nz   +
-          node[e_enqire[i].nod[12+n]].nz+node[e_enqire[i].nod[ 9+n]].nz) ;
-
-        e_enqire[i].nod[n+15]=node[anz->n].nr;
-        anz->n++;
-      }
-        anz->nmax++;
-        node[anz->n].nr = anz->nmax;
-        node[anz->nmax].pflag=1;
-        node[anz->nmax].indx=anz->n;
-
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[2]].nx+node[e_enqire[i].nod[0]].nx    +
-          node[e_enqire[i].nod[3]].nx+node[e_enqire[i].nod[5]].nx )  + 1./2.*(
-          node[e_enqire[i].nod[ 8]].nx+node[e_enqire[i].nod[ 9]].nx   +
-          node[e_enqire[i].nod[14]].nx+node[e_enqire[i].nod[11]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[2]].ny+node[e_enqire[i].nod[0]].ny    +
-          node[e_enqire[i].nod[3]].ny+node[e_enqire[i].nod[5]].ny )  + 1./2.*(
-          node[e_enqire[i].nod[ 8]].ny+node[e_enqire[i].nod[ 9]].ny   +
-          node[e_enqire[i].nod[14]].ny+node[e_enqire[i].nod[11]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[2]].nz+node[e_enqire[i].nod[0]].nz    +
-          node[e_enqire[i].nod[3]].nz+node[e_enqire[i].nod[5]].nz )  + 1./2.*(
-          node[e_enqire[i].nod[ 8]].nz+node[e_enqire[i].nod[ 9]].nz   +
-          node[e_enqire[i].nod[14]].nz+node[e_enqire[i].nod[11]].nz) ;
-
-        e_enqire[i].nod[17]=node[anz->n].nr;
-        anz->n++;
-
-        anz->nmax++;
-        node[anz->n].nr = anz->nmax;
-        node[anz->nmax].pflag=1;
-        node[anz->nmax].indx=anz->n;
-
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[0]].nx+node[e_enqire[i].nod[2]].nx    +
-          node[e_enqire[i].nod[1]].nx+node[e_enqire[i].nod[0]].nx )  + 1./2.*(
-          node[e_enqire[i].nod[ 8]].nx+node[e_enqire[i].nod[ 7]].nx   +
-          node[e_enqire[i].nod[ 6]].nx+node[e_enqire[i].nod[ 0]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[0]].ny+node[e_enqire[i].nod[2]].ny    +
-          node[e_enqire[i].nod[1]].ny+node[e_enqire[i].nod[0]].ny )  + 1./2.*(
-          node[e_enqire[i].nod[ 8]].ny+node[e_enqire[i].nod[ 7]].ny   +
-          node[e_enqire[i].nod[ 6]].ny+node[e_enqire[i].nod[ 0]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[0]].nz+node[e_enqire[i].nod[2]].nz    +
-          node[e_enqire[i].nod[1]].nz+node[e_enqire[i].nod[0]].nz )  + 1./2.*(
-          node[e_enqire[i].nod[ 8]].nz+node[e_enqire[i].nod[ 7]].nz   +
-          node[e_enqire[i].nod[ 6]].nz+node[e_enqire[i].nod[ 0]].nz) ;
-
-        e_enqire[i].nod[18]=node[anz->n].nr;
-        anz->n++;
-
-        anz->nmax++;
-        node[anz->n].nr = anz->nmax;
-        node[anz->nmax].pflag=1;
-        node[anz->nmax].indx=anz->n;
-
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[3]].nx+node[e_enqire[i].nod[4]].nx    +
-          node[e_enqire[i].nod[5]].nx+node[e_enqire[i].nod[3]].nx )  + 1./2.*(
-          node[e_enqire[i].nod[12]].nx+node[e_enqire[i].nod[13]].nx   +
-          node[e_enqire[i].nod[14]].nx+node[e_enqire[i].nod[ 3]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[3]].ny+node[e_enqire[i].nod[4]].ny    +
-          node[e_enqire[i].nod[5]].ny+node[e_enqire[i].nod[3]].ny )  + 1./2.*(
-          node[e_enqire[i].nod[12]].ny+node[e_enqire[i].nod[13]].ny   +
-          node[e_enqire[i].nod[14]].ny+node[e_enqire[i].nod[ 3]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[3]].nz+node[e_enqire[i].nod[4]].nz    +
-          node[e_enqire[i].nod[5]].nz+node[e_enqire[i].nod[3]].nz )  + 1./2.*(
-          node[e_enqire[i].nod[12]].nz+node[e_enqire[i].nod[13]].nz   +
-          node[e_enqire[i].nod[14]].nz+node[e_enqire[i].nod[ 3]].nz) ;
-
-        e_enqire[i].nod[19]=node[anz->n].nr;
-        anz->n++;
-    }
-    if (e_enqire[i].type == 10)
-    {
-      /* calculate the midside node */
-      if ( (node = (Nodes *)realloc( (Nodes *)node, (anz->nmax+2) * sizeof(Nodes))) == NULL )
-        printf("\n\n ERROR: realloc failed node\n\n") ;
-      anz->nmax++;
-      node[anz->n].nr = anz->nmax;
-      node[anz->nmax].pflag=1;
-      node[anz->nmax].indx=anz->n;
-
-        node[anz->nmax].nx = -1./4.* (
-          node[e_enqire[i].nod[0]].nx+node[e_enqire[i].nod[1]].nx  +
-          node[e_enqire[i].nod[3]].nx+node[e_enqire[i].nod[2]].nx )+ 1./2.*(
-          node[e_enqire[i].nod[4]].nx+node[e_enqire[i].nod[6]].nx  +
-          node[e_enqire[i].nod[7]].nx+node[e_enqire[i].nod[5]].nx) ;
-
-        node[anz->nmax].ny = -1./4.* (
-          node[e_enqire[i].nod[0]].ny+node[e_enqire[i].nod[1]].ny  +
-          node[e_enqire[i].nod[3]].ny+node[e_enqire[i].nod[2]].ny )+ 1./2.*(
-          node[e_enqire[i].nod[4]].ny+node[e_enqire[i].nod[6]].ny  +
-          node[e_enqire[i].nod[7]].ny+node[e_enqire[i].nod[5]].ny) ;
-
-        node[anz->nmax].nz = -1./4.* (
-          node[e_enqire[i].nod[0]].nz+node[e_enqire[i].nod[1]].nz  +
-          node[e_enqire[i].nod[3]].nz+node[e_enqire[i].nod[2]].nz )+ 1./2.*(
-          node[e_enqire[i].nod[4]].nz+node[e_enqire[i].nod[6]].nz  +
-          node[e_enqire[i].nod[7]].nz+node[e_enqire[i].nod[5]].nz) ;
-        e_enqire[i].nod[8]=node[anz->n].nr;
-        anz->n++;
-    }
-  }
   anz->nnext=anz->nmax+1;
 
   getElemNormalen( e_enqire, node, anz->e );
