@@ -70,10 +70,10 @@ double angleSurfBody( double *v0, double *v1, double *v2, double *scg, double *b
 /* lcmb rueckgabe von 1. linie bis n.linie orientiert (linie*ori zeigt in richtung letzte linie) */
 int orientLcmb( int nr )
 {
-  register int i,j;
+  int i,j;
   int     flag1, flag2, pnt=0, err=0, lcount=0;
-  static int     *sortLines=NULL;
-  static char    *sortOri=NULL;
+  int     *sortLines=NULL;
+  char    *sortOri=NULL;
 
   if(printFlag) printf (" orient lcmb:%s ", lcmb[nr].name);
 
@@ -120,7 +120,7 @@ int orientLcmb( int nr )
   }
   if (i==lcmb[nr].nl)
   {
-    errMsg(" ERROR: lcmb %s are a closed loop, no orientation possible\n", lcmb[nr].name);
+    errMsg(" ERROR: lcmb %s is a closed loop, no orientation possible\n", lcmb[nr].name);
     for (j=0; j<lcmb[nr].nl; j++) printf(" %s", line[lcmb[nr].l[j]].name);
     printf("\n");
     err=-i;
@@ -188,6 +188,8 @@ int orientLcmb( int nr )
   printf ("p1:%s p2:%s\n", point[lcmb[nr].p1].name, point[lcmb[nr].p2].name ); 
   */
 
+  free(sortLines);
+  free(sortOri);
   return(err);
 }
 
@@ -198,16 +200,23 @@ int orientLcmb( int nr )
 /* vorher ist ein tet-volumen netz nicht erzeugbar */
 int orientSurf( int nr )
 {
+#define TEST 0
   int i,j=0, n=0;
-  int     flag1, flag2, err=0, lcount=0;
-  int     *sortLines=NULL;
+  int     flag1, flag2, err=0, lcount=0, lastlcount, closedLoopFlag;
   int     p1,p2, pnt=0, sl, cl, l0;
   int     line0_p1, line0_p2;
-  static char    *sortOri=NULL, *tmpOri=NULL, *sortTyp=NULL;
-  int *cp=NULL;
+  int     *sortLines=NULL;
+  int     *cp=NULL;
+  char    *sortOri=NULL, *tmpOri=NULL, *sortTyp=NULL;
 
-
-  if(printFlag) printf (" orient surf:%s ", surf[nr].name);
+  if(surf[nr].nl<1) return(-1);
+  if(printFlag) printf (" orient surf:%s \n", surf[nr].name);
+#if TEST    
+  printf (" orient surf:%s nl:%d\n", surf[nr].name, surf[nr].nl);
+    for(i=0; i<surf[nr].nl; i++)
+      printf ("  surf[nr].o[i]:%c surf[nr].typ[n]:%c surf[nr].l[n]:%d\n", surf[nr].o[i],surf[nr].typ[n],surf[nr].l[n]);
+     
+#endif
 
   if((sortOri = (char *)realloc((char *)sortOri, (surf[nr].nl)*sizeof(char)) ) == NULL )
   { printf(" ERROR: realloc failure in orientSurf()\n\n"); return(-1); }
@@ -230,12 +239,13 @@ int orientSurf( int nr )
 
 
   /* Definiere die Linienzuege (curves) zb: Auessere Umrandung, innere Loecher */
-  /* die einzelnen Linenzuege werden zum NURBS-Trimming benutzt. Zumstrukturierten Vernetzen sind */
+  /* die einzelnen Linenzuege werden zum NURBS-Trimming benutzt. Zum strukturierten Vernetzen sind */
   /* nur Surfaces mit einem Linienzug geeignet */
   do
   {
+    lastlcount=lcount;
 #if TEST    
-    printf("curve starts with n:%d lcount:%d \n",n,lcount);
+    printf("\n curve starts with n:%d lcount:%d \n",n,lcount);
 #endif
     /* suche den punkt einer linie die mit der ersten linie verbunden ist */
     /* und orientiere die erste linie */
@@ -258,7 +268,7 @@ int orientSurf( int nr )
     }
   
 #if TEST    
-    for (i=0; i<surf[nr].nl; i++)
+    for (i=n; i<surf[nr].nl; i++)
     {
       if (surf[nr].typ[i]=='c')
         printf("%1c %1c %s\n",surf[nr].typ[i], surf[nr].o[i], lcmb[surf[nr].l[i]].name);
@@ -294,7 +304,7 @@ int orientSurf( int nr )
         p1=line[surf[nr].l[i]].p1;
         p2=line[surf[nr].l[i]].p2;
 #if TEST    
-    printf ("l:%s  \n", line[surf[nr].l[i]].name );
+    printf ("l:%s  p1:%s p2:%s  \n", line[surf[nr].l[i]].name, point[p1].name, point[p2].name );
 #endif
       }
   
@@ -346,12 +356,20 @@ int orientSurf( int nr )
 
     if (i==surf[nr].nl)
     {
-      errMsg("WARNING: surf not closed, no orientation possible\n");
-      for (i=0; i<surf[nr].nl; i++)
+      if(surf[nr].typ[n]=='l') errMsg("WARNING: line[%d]:%s not connected\n",n,line[surf[nr].l[n]].name);
+      //surf[nr].typ[n]='d';
+      if(surf[nr].typ[n]=='l')  surf[nr].typ[n]='L'; else surf[nr].typ[n]='C';
+      n++;
+      if(n==surf[nr].nl)
       {
-        surf[nr].o[i]='+';
+        errMsg("WARNING: surf not closed, no orientation possible\n");
+        for (i=0; i<surf[nr].nl; i++)
+        {
+          surf[nr].o[i]='+';
+        }
+        goto orientSurfError;
       }
-      return(-1);
+      else continue;
     }
     else
     {
@@ -366,16 +384,18 @@ int orientSurf( int nr )
     }
 
 #if TEST    
-   printf ("Curve:%d Start-line:%s ori:%c 2line:%s ori:%c \n",surf[nr].nc,line[surf[nr].l[n]].name, surf[nr].o[n], line[surf[nr].l[i]].name, surf[nr].o[i] );
+    printf ("Curve:%d Start-line:%s ori:%c 2line:%s ori:%c \n",surf[nr].nc,line[surf[nr].l[n]].name, surf[nr].o[n], line[surf[nr].l[i]].name, surf[nr].o[i] );
 #endif
-    for (i=n+1; i<surf[nr].nl; i++)  /* ori die restlichen linien */
+    
+    closedLoopFlag=0;
+    for (i=0; i<surf[nr].nl; i++) /* suche die anschliessende durch vergleich (with inner second loop!)*/
     {
-      for (j=0; j<surf[nr].nl; j++) /* suche die anschliessende durch vergleich */
+      for (j=n+1; j<surf[nr].nl; j++) /* suche die anschliessende durch vergleich */
       {
         if (!surf[nr].o[j])              /* wenn ori noch == 0 */
         {
 #if TEST    
-          printf ("pnt:%s nr:%d j:%d\n", point[pnt].name, nr,j);
+          printf ("check pnt:%s pnt:%d j:%d\n", point[pnt].name, pnt, j);
 #endif
           if (surf[nr].typ[j]=='c')
           {
@@ -386,7 +406,7 @@ int orientSurf( int nr )
             if(lcmb[sl].o[cl]=='+')   p2=line[lcmb[sl].l[cl]].p2;
             else                      p2=line[lcmb[sl].l[cl]].p1;
 #if TEST    
-   printf ("lcmb:%s pnt:%d p1:%d p2:%d\n", lcmb[surf[nr].l[j]].name, pnt, p1, p2 );
+          printf ("compare i:%d j:%d lcmb:%s pnt:%d p1:%d p2:%d\n", i,j, lcmb[surf[nr].l[j]].name, pnt, p1, p2 );
 #endif
           }
           else
@@ -394,76 +414,103 @@ int orientSurf( int nr )
             p1=line[surf[nr].l[j]].p1;
             p2=line[surf[nr].l[j]].p2;
 #if TEST    
-   printf ("i:%d j:%d line:%s ori:%c pnt:%d p1:%d p2:%d\n", i,j,line[surf[nr].l[j]].name, surf[nr].o[j], pnt, p1, p2 );
+          printf ("compare i:%d j:%d line:%s ori:%c pnt:%d p1:%d p2:%d\n", i,j, surf[nr].o[j], pnt, p1, p2 );
 #endif
           }
-  
+
           if ( p1 == pnt )
-          {  pnt=p2; surf[nr].o[j]='+';
+          {
+	    pnt=p2; surf[nr].o[j]='+';
 #if TEST    
-   printf ("i:%d j:%d line:%s pnt:%d p1:%d p2:%d\n", i,j,line[surf[nr].l[j]].name, pnt, p1, p2 );
+          printf ("found i:%d j:%d p1:%d p2:%d  next pnt:%d\n", i,j, p1, p2, pnt );
 #endif
-             sortLines[lcount]=surf[nr].l[j]; sortOri[lcount]=surf[nr].o[j];
-             sortTyp[lcount]=surf[nr].typ[j]; lcount++; }
+            sortLines[lcount]=surf[nr].l[j]; sortOri[lcount]=surf[nr].o[j];
+            sortTyp[lcount]=surf[nr].typ[j]; lcount++;
+          }
           else if ( p2 ==  pnt )
-          {  pnt=p1; surf[nr].o[j]='-';
+          {
+	    pnt=p1; surf[nr].o[j]='-';
 #if TEST    
-   printf ("i:%d j:%d line:%s pnt:%d p1:%d p2:%d\n", i,j,line[surf[nr].l[j]].name, pnt, p1, p2 );
+          printf ("found i:%d j:%d  p1:%d p2:%d  next pnt:%d\n", i,j, p1, p2, pnt );
 #endif
-             sortLines[lcount]=surf[nr].l[j]; sortOri[lcount]=surf[nr].o[j];
-             sortTyp[lcount]=surf[nr].typ[j]; lcount++; }
+            sortLines[lcount]=surf[nr].l[j]; sortOri[lcount]=surf[nr].o[j];
+            sortTyp[lcount]=surf[nr].typ[j]; lcount++;
+          }
         }
       }
-    }
 
-    /* check if the curve is closed and store the first line-index of the surf */
-    if(sortTyp[n]=='l')
-    {
-      if((pnt==line[sortLines[n]].p1)||(pnt==line[sortLines[n]].p2))
-      {
-        /* start-index of the curve */
-        if((surf[nr].c=(int *)realloc((int *)surf[nr].c, (surf[nr].nc+1)*sizeof(int)) )==NULL)
-        { printf(" ERROR: realloc failure in orient surf:%s can not be oriented\n", surf[nr].name); exit(-1); }
-        surf[nr].c[surf[nr].nc]=lcount-n;
-        surf[nr].nc++;
-      }
-    }
-    else if(sortTyp[n]=='c')
-    {
-      if((pnt==lcmb[sortLines[n]].p1)||(pnt==lcmb[sortLines[n]].p2))
-      {
-        /* start-index of the curve */
-        if((surf[nr].c=(int *)realloc((int *)surf[nr].c, (surf[nr].nc+1)*sizeof(int)) )==NULL)
-        { printf(" ERROR: realloc failure in orient surf:%s can not be oriented\n", surf[nr].name); exit(-1); }
-        surf[nr].c[surf[nr].nc]=lcount-n;
-        surf[nr].nc++;
-      }
-    }
-    else
-    {
-      printf("WARNING in ori: Surf:%s has an unclosed curve\n", surf[nr].name);
-      printf("typ:%c pnt:%s line[sortLines[n]].p1:%s line[sortLines[n]].p2:%s\n", sortTyp[n], point[pnt].name, point[line[sortLines[n]].p1].name, point[line[sortLines[n]].p2].name);
-      goto finish_ori;
-    }
+#if TEST    
+      printf ("check connected lines\n");
+#endif
 
+      /* check if the curve is closed  */
+      if(surf[nr].typ[n]=='l')
+      {
+        if((pnt==line[surf[nr].l[n]].p1)||(pnt==line[surf[nr].l[n]].p2))
+        {
+          /* number of lines in each curve */
+          if((surf[nr].c=(int *)realloc((int *)surf[nr].c, (surf[nr].nc+1)*sizeof(int)) )==NULL)
+          { printf(" ERROR: realloc failure in orient surf:%s can not be oriented\n", surf[nr].name); exit(-1); }
+          surf[nr].c[surf[nr].nc]=lcount-lastlcount;
+          surf[nr].nc++;
+	  closedLoopFlag=1;
+#if TEST    
+          printf ("line, closed line loop found, n:%d\n",n);
+#endif
+        }
+      }
+      else if(surf[nr].typ[n]=='c')
+      {
+        if((pnt==lcmb[surf[nr].l[n]].p1)||(pnt==lcmb[surf[nr].l[n]].p2))
+        {
+          /* number of lines in each curve */
+          if((surf[nr].c=(int *)realloc((int *)surf[nr].c, (surf[nr].nc+1)*sizeof(int)) )==NULL)
+          { printf(" ERROR: realloc failure in orient surf:%s can not be oriented\n", surf[nr].name); exit(-1); }
+          surf[nr].c[surf[nr].nc]=lcount-lastlcount;
+          surf[nr].nc++;
+	  closedLoopFlag=1;
+#if TEST    
+          printf ("lcmb, closed line loop found, n:%d\n",n);
+#endif
+        }
+      }
+      if(closedLoopFlag==1) break;
+    }
+    if(closedLoopFlag==0) lcount=lastlcount;
+
+    
     /* check for unoriented lines */
     err=0;
     for (n=0; n<surf[nr].nl; n++)
     {
-      if (!surf[nr].o[n])
-      {
+      //if( (!surf[nr].o[n])&&(surf[nr].typ[n]!='d'))
+       if( (!surf[nr].o[n]) && ((surf[nr].typ[n]!='L')&&(surf[nr].typ[n]!='C')) )
+       {
         err--;
         break;
       }
     }
   }while(err<0);
 
+#if TEST    
+  printf("lcount:%d surf[nr].nl:%d\n",lcount,surf[nr].nl);
+#endif
+
+  /* re-define surf[nr] here and erase the unconnected lines */
+  if(!lcount) goto orientSurfError;
+  for (n=0; n<lcount; n++)
+  {
+      surf[nr].typ[n]=sortTyp[n];
+      surf[nr].l[n]=sortLines[n];
+      surf[nr].o[n]=sortOri[n];
+  }
+  surf[nr].nl=lcount;
+  
 
-  finish_ori:;
   /* check of the loop was closed */
   if(surf[nr].nc==0)
   {
-    errMsg(" ERROR: surf:%s not chained and therefore not oriented.\n", surf[nr].name );
+    errMsg(" WARNING: surf:%s not chained and therefore not oriented.\n", surf[nr].name );
     err--;
   }
 
@@ -567,12 +614,32 @@ int orientSurf( int nr )
   surf[nr].cp=cp;
   for (i=0; i<surf[nr].nl; i++)
   {  
+    if(surf[nr].typ[i]=='L') surf[nr].typ[i]='l';
+    if(surf[nr].typ[i]=='C') surf[nr].typ[i]='c'; // back to original typ
     surf[nr].cx+=(point[cp[i]].px* scale->w+scale->x)/surf[nr].nl;
     surf[nr].cy+=(point[cp[i]].py* scale->w+scale->y)/surf[nr].nl;
     surf[nr].cz+=(point[cp[i]].pz* scale->w+scale->z)/surf[nr].nl;
   }
-
+  free(sortLines);
+  free(cp);
+  free(sortOri);
+  free(tmpOri);
+  free(sortTyp);
+  surf[nr].fail=0;
   return(err);
+ orientSurfError:;
+  for (i=0; i<surf[nr].nl; i++)
+  {
+    if(surf[nr].typ[i]=='L')  surf[nr].typ[i]='l';
+    if(surf[nr].typ[i]=='C')  surf[nr].typ[i]='c'; // back to original typ
+  }
+  free(sortLines);
+  free(cp);
+  free(sortOri);
+  free(tmpOri);
+  free(sortTyp);
+  surf[nr].fail=2 ;
+  return(-1);
 }
 
 
@@ -640,7 +707,7 @@ int ori7SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -672,7 +739,7 @@ int ori7SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -704,7 +771,7 @@ int ori7SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -735,7 +802,7 @@ int ori7SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -766,7 +833,7 @@ int ori7SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -964,6 +1031,7 @@ int ori6SidedBody( int nr, int **cp )
   n=0;
   for (i=0; i<body[nr].ns; i++)
   {
+    if (surf[body[nr].s[i]].nl<4) return(-1);
     if (cp[i][0]==cp[i][1]) {  p_colapse[n]=cp[i][0]; n++; }
     if (cp[i][1]==cp[i][2]) {  p_colapse[n]=cp[i][1]; n++; }
     if (cp[i][2]==cp[i][3]) {  p_colapse[n]=cp[i][2]; n++; }
@@ -1016,7 +1084,7 @@ int ori6SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -1051,7 +1119,7 @@ int ori6SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -1085,7 +1153,7 @@ int ori6SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -1119,7 +1187,7 @@ int ori6SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -1149,7 +1217,7 @@ int ori6SidedBody( int nr, int **cp )
     if(!cl) break;
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
   surfIndex[k]=s;
@@ -1366,7 +1434,7 @@ int ori5SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -1398,7 +1466,7 @@ int ori5SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -1429,7 +1497,7 @@ int ori5SidedBody( int nr, int **cp )
     }
   }
   if (i>=body[nr].ns)
-  { errMsg (" ERROR in orientBody, side %d from body:%s could not be located \n",
+  { errMsg (" WARNING in orientBody, side %d from body:%s could not be located \n",
       k, body[nr].name); return(-1);
   }
 #if TEST
@@ -1569,14 +1637,14 @@ int ori5SidedBody( int nr, int **cp )
 int oriTetBody( int nr, int **cp )
 {
   int i,j,k,n,l,ll,lll,cl,s,sb, prod1, prod2,oriflag,counter=0,surl,sur, sl=0, p1,p2;
-  int **ltos, *sori;
+  int **ltos, *sori=NULL;
   int *surfOriBuffer=NULL;
   double scg[3], bcg[3];
   double v0[3], v1[3], v2[3];
   double fi, dist;
-  static double *max_dist=NULL;
-  static int *i_ind=NULL;
   int dist_count=0;
+  double *max_dist=NULL;
+  int *i_ind=NULL;
 
   /* go over all surfs of that body*/
   /* check if one neighbour surf is oriented */
@@ -1613,7 +1681,7 @@ int oriTetBody( int nr, int **cp )
         {
           printf("ERROR: to many related surfs(%d) for line:%s\n", n, line[surf[s].l[j]].name);
           printf("No inner surfaces are permitted. Command could not be executed\n");
-          return(-1);
+          goto orientTetBodyError;
         }
         ltos[surf[s].l[j]][n]=s;
       }
@@ -1627,7 +1695,7 @@ int oriTetBody( int nr, int **cp )
           {
             printf("ERROR: to many related surfs(%d) for line:%s\n", n, line[lcmb[cl].l[l]].name);
             printf("No inner surfaces are permitted. Command could not be executed\n");
-            return(-1);
+            goto orientTetBodyError;
           }
           ltos[lcmb[cl].l[l]][n]=s;
 	}
@@ -1776,7 +1844,7 @@ int oriTetBody( int nr, int **cp )
   {
     counter++;
     if(counter<body[nr].ns) goto more;
-    else { printf(" WARNING: too much loops. Some surfs might be still unoriented.\n"); return(1); }
+    else { printf(" WARNING: too many loops. Some surfs might be still unoriented.\n"); goto orientTetBodyError; }
   }
 
   /* die orientierungen aller surfs werden nun mit hilfe des schwerpunktes des bodies ueberprueft */
@@ -1864,14 +1932,27 @@ int oriTetBody( int nr, int **cp )
     if(printFlag) printf (" %1c %s", body[nr].o[i], surf[ body[nr].s[i] ].name );
   }
   if(printFlag) printf("\n");
+
+  for(i=0; i<anzGeo->l; i++) free(ltos[i]);
+  free(ltos);
+  free(sori);
+  free(max_dist);
+  free(i_ind);
   return(1);
+ orientTetBodyError:;
+  for(i=0; i<anzGeo->l; i++) free(ltos[i]);
+  free(ltos);
+  free(sori);
+  free(max_dist);
+  free(i_ind);
+  return(-1);
 }
 
 
 
 int orientBody( int nr )
 {
-  register int i,j=0,n;
+  int i,j=0,n;
   int   s, sl, cl, err=0;
   int   **cp;                  /* corner points of the surfs */
 
@@ -1879,7 +1960,7 @@ int orientBody( int nr )
   { printf(" ERROR: malloc failure in orientBody()\n"); return(-1); }
   for(i=0; i<body[nr].ns; i++)
   {
-    if( (cp[i]=(int *)malloc(surf[body[nr].s[i]].nl*sizeof(int) ) )==NULL)    
+    if( (cp[i]=(int *)calloc(surf[body[nr].s[i]].nl+4, sizeof(int) ) )==NULL)    
     { printf(" ERROR: malloc failure in orientBody()\n"); return(-1); }
   }
 
@@ -1891,6 +1972,10 @@ int orientBody( int nr )
   for (i=0; i<body[nr].ns; i++)
   {
     s=body[nr].s[i];
+    if ( orientSurf( s ) <0 )
+    {
+      errMsg ("WARNING: orientSurf:%s failed\n", surf[s].name);
+    }
 
     /* suche den ersten punkt der ersten linie oder lcmb der surf */
     if(surf[s].nl<1)
@@ -1921,7 +2006,7 @@ int orientBody( int nr )
         cp[i][0]=line[surf[s].l[0]].p1;
       }
     }
-    else { errMsg (" ERROR in orientBody, surf.typ:%1c not known\n", surf[s].typ[j]); exit(-1);}
+    else { errMsg (" WARNING in orientBody, surf.typ:%1c not known\n", surf[s].typ[j]); exit(-1);}
 
     /* suche den anfangspunkt aller weiteren linien oder lcmbs der surf */
     for (j=1; j<surf[s].nl; j++)
@@ -1952,7 +2037,7 @@ int orientBody( int nr )
           cp[i][j]=line[surf[s].l[j]].p1;
         }
       }
-      else { errMsg (" ERROR in orientBody, surf.typ:%1c not known\n", surf[s].typ[j]); exit(-1);}
+      else { errMsg (" WARNING in orientBody, surf.typ:%1c not known\n", surf[s].typ[j]); exit(-1);}
     }
     /*
     printf ("\n");
@@ -1986,7 +2071,7 @@ int orientBody( int nr )
 
   /* die Eckknoten aller Surfaces sind sortiert, sortiere und orientiere die surfaces */
   /* entsprechend den Anforderungen des Meshers */
-
+  err=1;
   if (body[nr].ns==6)
   { if(ori6SidedBody( nr, cp )==-1) err=-2;}
   else if (body[nr].ns==5)
@@ -1997,6 +2082,8 @@ int orientBody( int nr )
   { if(oriTetBody( nr, cp )==-1) err=-1;}
   else err=0;
 
+  // ignore the failed orientation since the body might still be tet meshable
+  if(err<0) err=0;
 
   for(i=0; i<body[nr].ns; i++) free(cp[i]);
   free(cp);
@@ -2007,7 +2094,7 @@ int orientBody( int nr )
 void orientSet( char *record)
 {
   int   setNr, flag;
-  register int i,j=0;
+  int i,j=0;
   char setname[MAX_LINE_LENGTH];
 
   sword( record, setname );
