@@ -206,7 +206,7 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
   int i, j, n, ld100, lfile;
   char rec_str[MAX_LINE_LENGTH];
   char buffer[MAX_LINE_LENGTH];
-  int  anz_n, time_step=0;
+  int  anz_n, time_step=1;          //time_step defines the first dunsmov.<time_step> file to read
   int  e_nmax=1, e_nmin=1;
   int  length;
   float fval[1];
@@ -225,10 +225,10 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
   struct utsname  cursys[1];
 
   uname (cursys);
-  if(compareStrings(cursys->machine, "x86_64")>0) { HEAD_CHARS=8; MID_CHARS=16; }
+  if(elem_type<0) { HEAD_CHARS=8; MID_CHARS=16; elem_type*=-1; }
+  //if(compareStrings(cursys->machine, "x86_64")>0) { HEAD_CHARS=8; MID_CHARS=16; }
   else  { HEAD_CHARS=4; MID_CHARS=8; }
   // in case the file does not read correctly (for example wrong nr of blocks are listed in the xterm) it might help to try the chars manually:
-  //HEAD_CHARS=4; MID_CHARS=8;
 
   if ( (lcase = (Datasets *)malloc( 1 * sizeof(Datasets))) == NULL )
     printf("\n\n ERROR: malloc failed, lcase\n\n") ;
@@ -264,7 +264,7 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
   {
     printf ("\nThe input file \"%s\" could not be opened.\n", datin);
     printf (" trying to use the file-extension \".1\"\n");
-    sprintf(&datin[lfile], ".1");
+    sprintf(&datin[lfile], ".%d", time_step);
     handle2 = fopen (datin, "rb");
     if ( handle2== NULL )
     { printf ("\nThe input file \"%s\" could also not be opened.\n\n",datin);
@@ -395,7 +395,8 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
             elem[anz->e].group= i;
             elem[anz->e].mat = 1;
             anz->etype[elem[anz->e].type]++;
-            elem[anz->e].nr  = ++anz->e;
+            anz->e++;
+            elem[anz->e-1].nr  = anz->e;
           }
         }
       }
@@ -478,7 +479,8 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
           elem[anz->e].group= i;
           elem[anz->e].mat = 1;
           anz->etype[elem[anz->e].type]++;
-          elem[anz->e].nr  = ++anz->e;
+          anz->e++;
+          elem[anz->e-1].nr  = anz->e;
         }
       }  
       anz->n+=nsz;
@@ -671,7 +673,7 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
       lcase[anz->l].value=ld100;
       lcase[anz->l].ncomps = nident;
       lcase[anz->l].irtype = 1;
-      sprintf( lcase[anz->l].name,"%s", datin);
+      sprintf( lcase[anz->l].name,"%s", "duns");
       printf ("lcase.name[%d]= %s\n", anz->l, lcase[anz->l].name);
 
       lcase[anz->l].npheader=0;
@@ -766,8 +768,8 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
     fclose (handle2);
 
     /* open the file of the next time-step if any */
-    time_step++;
     sprintf(&datin[lfile], ".%d", time_step);
+    time_step++;
     handle2 = fopen (datin, "rb");
     if ( handle2== NULL )
     { printf ("\nThe input file \"%s\" could not be opened.\n\n", datin); break; }
@@ -781,12 +783,12 @@ int readDuns(char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
 
   if ( e_nmax > (anz->nmax) )
   {
-    printf ("\n WARNING: element requestes a nodename higher than allocated\n\n");
+    printf ("\n WARNING: element requests a nodename higher than allocated\n\n");
     printf (" e_nmax=%d e_nmin=%d \n", e_nmax, e_nmin );
   }
   if ( e_nmin < 1 )
   {
-    printf ("\n WARNING: element requestes a nodename lower than allocated\n\n");
+    printf ("\n WARNING: element requests a nodename lower than allocated\n\n");
     printf (" e_nmax=%d e_nmin=%d \n", e_nmax, e_nmin );
   }
 
