@@ -33,7 +33,7 @@
 
 void freeDataset(Datasets *lcase, int nr)
 {
-  register int i;
+  int i;
 
   printf(" free lc[%d] ncomps:%d\n",nr,lcase[nr].ncomps);
   if(lcase[nr].loaded)
@@ -85,7 +85,7 @@ void freeDataset(Datasets *lcase, int nr)
 int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets **lptr, int read_mode )
 {
   FILE *handle;
-  int i=0, j=0;
+  int i=0, j=0, k;
   int  nodeflag=0, elemflag=0, errFlag=0, firsttime=1;
   int n;  /* used in format_flag */
   long offset=0;
@@ -99,6 +99,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
   int  length, flag, format_flag;
   int  ipuf, nodenr=0;
   static float *value=NULL;
+  static double *dvalue=NULL;
 
   char **dat, **compName;
   int          *menu, *ictype, *icind1, *icind2, *iexist; 
@@ -121,11 +122,12 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
   format_flag=0;
 
   /* Open the files and check to see that it was opened correctly */
-  handle = fopen (datin, "r");
+  handle = fopen (datin, "rb");
   if ( handle== NULL )  { printf ("ERROR in readfrd: The input file \"%s\" could not be opened.\n\n", datin); return(-1); }
   else  printf (" file:%s opened\n", datin);
 
-
+  anz->filehandle=handle;
+  
   printf (" reading frd format\n");
   length = frecord( handle, rec_str);
   rec_str[length]='\0';
@@ -158,7 +160,8 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
 
 
     flag = stoi(rec_str,1,5);
-    format_flag = stoi(rec_str,74,75);
+    if(strlen(rec_str)>73) format_flag = stoi(rec_str,74,75);
+    else format_flag = 0;
     //printf ("OPCODE:%d IFORMT:%d\n", flag, format_flag );
 
     if(flag == 9999) goto read_again;
@@ -327,6 +330,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
             node[node[anz->n].nr].ny = value[1];
             node[node[anz->n].nr].nz = value[2];
           }
+          else fseeko(handle, 3*sizeof(float), SEEK_CUR);
 	}
         else
 	{
@@ -335,7 +339,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
             length=fread((double *)&node[node[anz->n].nr].nx,sizeof(double),3,handle);
             node[node[anz->n].nr].indx=anz->n;
           }
-          else fseek(handle, 3*sizeof(double), SEEK_CUR);
+          else fseeko(handle, 3*sizeof(double), SEEK_CUR);
 	}
         if (node[anz->n].nr >  anz->nmax)  anz->nmax=node[anz->n].nr;
         if (node[anz->n].nr <  anz->nmin)  anz->nmin=node[anz->n].nr;
@@ -569,6 +573,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
         printf("nods in block assumed:%d\n",nod_data );
       }
 #endif
+      lcase[anz->l].numnod=nod_data;
       stos( rec_str, 37, 56, lcase[anz->l].dataset_text);
       lcase[anz->l].analysis_type=stoi( rec_str, 57, 58 );
       lcase[anz->l].step_number=stoi( rec_str, 59, 63 );
@@ -613,7 +618,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
       do
       {
         /* bin mode, active after last column definition was read (-5 lines). Attention flag:-6 not permitted so far! */
-        if (( format_flag==2)&&(lcase[anz->l].ncomps>0)&&(ncomps==lcase[anz->l].ncomps))
+        if (( format_flag>=2)&&(lcase[anz->l].ncomps>0)&&(ncomps==lcase[anz->l].ncomps))
         {
           //printf("format_flag=%d ncomps:%d lcncomps:%d\n",format_flag,ncomps,lcase[anz->l].ncomps);
 
@@ -631,22 +636,43 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
               lcase[anz->l].handle=handle;
               strcpy(lcase[anz->l].filename,datin);
 
-              if( fseek( handle, offset, SEEK_CUR )!=0) printf("error in fseek\n");
+              if( fseeko( handle, offset, SEEK_CUR )!=0) printf("error in fseeko\n");
 	    }
 	  }
-          else
+          else if(format_flag==2)
 	  {
+	    ncomps=0;
+	    for(i=0; i<lcase[anz->l].ncomps; i++) if(lcase[anz->l].iexist[i]!=1) ncomps++;
             if ( (value = (float *)realloc((float *)value, (lcase[anz->l].ncomps) * sizeof(float))) == NULL )
               printf("\n\n ERROR: realloc failed, value\n\n") ;
             for(n=0; n<nod_data; n++)
             {
               length=fread((int *)&nodenr,sizeof(int),1,handle);
-              length=fread((float *)value,sizeof(float),lcase[anz->l].ncomps,handle);
+              length=fread((float *)value,sizeof(float),ncomps,handle);
+	      // printf("n:%d N:%d ",n+1, nodenr);
+	      k=0;
+              for(i=0; i<lcase[anz->l].ncomps; i++) if(lcase[anz->l].iexist[i]!=1)
+              {
+	        // printf(" %f",value[k]); 
+                lcase[anz->l].dat[i][nodenr]= value[k];
+		k++;
+              }
+              // printf("\n");
+            }
+	  }
+          else if(format_flag==3)
+	  {
+            if ( (dvalue = (double *)realloc((double *)dvalue, (lcase[anz->l].ncomps) * sizeof(double))) == NULL )
+              printf("\n\n ERROR: realloc failed, dvalue\n\n") ;
+            for(n=0; n<nod_data; n++)
+            {
+              length=fread((int *)&nodenr,sizeof(int),1,handle);
+              length=fread((double *)dvalue,sizeof(double),lcase[anz->l].ncomps,handle);
 	      // printf("n:%d N:%d ",n+1, nodenr); 
               for(i=0; i<lcase[anz->l].ncomps; i++)
               {
-	        // printf(" %f",value[i]); 
-                lcase[anz->l].dat[i][nodenr]= value[i];
+	        // printf(" %f",dvalue[i]); 
+                lcase[anz->l].dat[i][nodenr]= dvalue[i];
               }
               // printf("\n");
             }
@@ -677,7 +703,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
               strcpy(lcase[anz->l].filename,datin);
 
               /* reduce the offset by the current record */
-              if( fseek( handle, offset-length, SEEK_CUR )!=0) printf("error in fseek\n");
+              if( fseeko( handle, offset-length, SEEK_CUR )!=0) printf("error in fseeko\n");
 	    }
 	  }
           else
@@ -754,9 +780,9 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
             printf("\n\n ERROR: malloc failure\n\n" );
           if ( (lcase[anz->l].nmin = (int *)malloc( (lcase[anz->l].ncomps) * sizeof(int))) == NULL )
             printf("\n\n ERROR: malloc failure\n\n" );
-          if ( (lcase[anz->l].max = (float *)malloc( (lcase[anz->l].ncomps) * sizeof(float))) == NULL )
+          if ( (lcase[anz->l].max = (CGXFLOAT *)malloc( (lcase[anz->l].ncomps) * sizeof(CGXFLOAT))) == NULL )
             printf("\n\n ERROR: malloc failure\n\n" );
-          if ( (lcase[anz->l].min = (float *)malloc( (lcase[anz->l].ncomps) * sizeof(float))) == NULL )
+          if ( (lcase[anz->l].min = (CGXFLOAT *)malloc( (lcase[anz->l].ncomps) * sizeof(CGXFLOAT))) == NULL )
             printf("\n\n ERROR: malloc failure\n\n" );
           if ( (lcase[anz->l].compName = (char **)malloc( (lcase[anz->l].ncomps) * sizeof(char *))) == NULL )
             printf("\n\n ERROR: malloc failure\n\n" );
@@ -772,9 +798,8 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
             printf("\n\n ERROR: malloc failure\n\n" );
           if ( (lcase[anz->l].iexist = (int *)malloc( lcase[anz->l].ncomps * sizeof(int))) == NULL )
             printf("\n\n ERROR: malloc failure\n\n" );
-          if ( (lcase[anz->l].dat = (float **)malloc( (lcase[anz->l].ncomps) * sizeof(float *))) == NULL )
+          if ( (lcase[anz->l].dat = (CGXFLOAT **)malloc( (lcase[anz->l].ncomps) * sizeof(CGXFLOAT *))) == NULL )
             printf("\n\n ERROR: malloc failure\n\n" );
-  printf(" gen lc[%d] ncomps:%d\n",anz->l,lcase[anz->l].ncomps);
           for(i=0; i<(lcase[anz->l].ncomps); i++)
 	  {
             if ( (lcase[anz->l].compName[i] = (char *)malloc( MAX_LINE_LENGTH * sizeof(char))) == NULL )
@@ -783,6 +808,8 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
                printf("\n\n ERROR: malloc failed\n\n" );
             lcase[anz->l].max[i]=-MAX_FLOAT;
             lcase[anz->l].min[i]=MAX_FLOAT;
+            lcase[anz->l].nmax[i]=0;
+            lcase[anz->l].nmin[i]=0;
 	  }
         }
         else if(flag == -5)
@@ -797,12 +824,14 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
             lcase[anz->l].iexist[indx] = stoi(rec_str,34,38);
 
             /* requests for additional components are not supported so far */
+	    /*
             if(lcase[anz->l].iexist[indx]==1)
 	    {
               free(lcase[anz->l].compName[indx]); lcase[anz->l].compName[indx]=NULL;
               free(lcase[anz->l].icname[indx]); lcase[anz->l].icname[indx]=NULL;
 	    }
-            else ncomps++;
+            else */
+	      ncomps++;
             indx++;
 	  }
           else
@@ -833,6 +862,10 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
 		{
                   offset= nod_data * (4+nvals*4); 
 		}
+                else if (format_flag==3)
+		{
+                  offset= nod_data * (4+nvals*8); 
+		}
                 else
 		{
                   /* just to get an approximate offset: */
@@ -857,7 +890,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
               /* in case the data should be read directly and not on demand */
               for(i=0; i<(lcase[anz->l].ncomps); i++)
 	      {
-                if ( (lcase[anz->l].dat[i] = (float *)malloc( (anz->nmax+1) * sizeof(float))) == NULL )
+                if ( (lcase[anz->l].dat[i] = (CGXFLOAT *)malloc( (anz->nmax+1) * sizeof(CGXFLOAT))) == NULL )
                   printf("\n\n ERROR: malloc failure\n\n" );	               
                 for(j=0; j<=anz->nmax; j++) lcase[anz->l].dat[i][j]=0.;
 	      }
@@ -917,7 +950,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
       {
        for(j=0; j<anz->n; j++)
        {
-        for(i=0; i<lcase[anz->l].ncomps; i++)
+	 for(i=0; i<lcase[anz->l].ncomps; i++) if(lcase[anz->l].iexist[i]!=1)
         {
           if(lcase[anz->l].dat[i][node[j].nr] > lcase[anz->l].max[i])
           {
@@ -952,11 +985,11 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
         printf("\n\n ERROR: malloc failure\n\n" );
       if ( (lcase[anz->l].nmin = (int *)malloc( lcase[anz->l].ncomps * sizeof(int))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].max = (float *)malloc( lcase[anz->l].ncomps * sizeof(float))) == NULL )
+      if ( (lcase[anz->l].max = (CGXFLOAT *)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].min = (float *)malloc( lcase[anz->l].ncomps * sizeof(float))) == NULL )
+      if ( (lcase[anz->l].min = (CGXFLOAT *)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].dat = (float **)malloc( lcase[anz->l].ncomps * sizeof(float *))) == NULL )
+      if ( (lcase[anz->l].dat = (CGXFLOAT **)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT *))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
       if ( (lcase[anz->l].compName = (char **)malloc( lcase[anz->l].ncomps * sizeof(char *))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
@@ -970,7 +1003,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
            printf("\n\n ERROR: malloc failed\n\n" );
         lcase[anz->l].max[i]=-MAX_FLOAT;
         lcase[anz->l].min[i]=MAX_FLOAT;
-        if ( (lcase[anz->l].dat[i] = (float *)malloc( (anz->nmax+1) * sizeof(float))) == NULL )
+        if ( (lcase[anz->l].dat[i] = (CGXFLOAT *)malloc( (anz->nmax+1) * sizeof(CGXFLOAT))) == NULL )
           printf("\n\n ERROR: malloc failure\n\n" );	               
         for(j=0; j<=anz->nmax; j++) lcase[anz->l].dat[i][j]=0.;
       }
@@ -1060,11 +1093,11 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
         printf("\n\n ERROR: malloc failure\n\n" );
       if ( (lcase[anz->l].nmin = (int *)malloc( lcase[anz->l].ncomps * sizeof(int))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].max = (float *)malloc( lcase[anz->l].ncomps * sizeof(float))) == NULL )
+      if ( (lcase[anz->l].max = (CGXFLOAT *)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].min = (float *)malloc( lcase[anz->l].ncomps * sizeof(float))) == NULL )
+      if ( (lcase[anz->l].min = (CGXFLOAT *)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].dat = (float **)malloc( lcase[anz->l].ncomps * sizeof(float *))) == NULL )
+      if ( (lcase[anz->l].dat = (CGXFLOAT **)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT *))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
       if ( (lcase[anz->l].compName = (char **)malloc( lcase[anz->l].ncomps * sizeof(char *))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
@@ -1072,7 +1105,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
         printf("\n\n ERROR: malloc failure\n\n" );
       for(i=0; i<lcase[anz->l].ncomps; i++)
       {
-        if ( (lcase[anz->l].dat[i] = (float *)malloc( (anz->nmax+1) * sizeof(float))) == NULL )
+        if ( (lcase[anz->l].dat[i] = (CGXFLOAT *)malloc( (anz->nmax+1) * sizeof(CGXFLOAT))) == NULL )
           printf("\n\n ERROR: malloc failure\n\n" );	               
         if ( (lcase[anz->l].compName[i] = (char *)malloc( MAX_LINE_LENGTH * sizeof(char))) == NULL )
            printf("\n\n ERROR: malloc failed\n\n" );
@@ -1080,6 +1113,8 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
            printf("\n\n ERROR: malloc failed\n\n" );
         lcase[anz->l].max[i]=-MAX_FLOAT;
         lcase[anz->l].min[i]=MAX_FLOAT;
+        lcase[anz->l].nmax[i]=0;
+        lcase[anz->l].nmin[i]=0;
         for(j=0; j<=anz->nmax; j++) lcase[anz->l].dat[i][j]=0.;
       }
       if ( (lcase[anz->l].menu = (int *)malloc( lcase[anz->l].ncomps * sizeof(int))) == NULL )
@@ -1187,11 +1222,11 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
         printf("\n\n ERROR: malloc failure\n\n" );
       if ( (lcase[anz->l].nmin = (int *)malloc( lcase[anz->l].ncomps * sizeof(int))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].max = (float *)malloc( lcase[anz->l].ncomps * sizeof(float))) == NULL )
+      if ( (lcase[anz->l].max = (CGXFLOAT *)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].min = (float *)malloc( lcase[anz->l].ncomps * sizeof(float))) == NULL )
+      if ( (lcase[anz->l].min = (CGXFLOAT *)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
-      if ( (lcase[anz->l].dat = (float **)malloc( lcase[anz->l].ncomps * sizeof(float *))) == NULL )
+      if ( (lcase[anz->l].dat = (CGXFLOAT **)malloc( lcase[anz->l].ncomps * sizeof(CGXFLOAT *))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
       if ( (lcase[anz->l].compName = (char **)malloc( lcase[anz->l].ncomps * sizeof(char *))) == NULL )
         printf("\n\n ERROR: malloc failure\n\n" );
@@ -1199,7 +1234,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
         printf("\n\n ERROR: malloc failure\n\n" );
       for(i=0; i<lcase[anz->l].ncomps; i++)
       {
-        if ( (lcase[anz->l].dat[i] = (float *)malloc( (anz->nmax+1) * sizeof(float))) == NULL )
+        if ( (lcase[anz->l].dat[i] = (CGXFLOAT *)malloc( (anz->nmax+1) * sizeof(CGXFLOAT))) == NULL )
           printf("\n\n ERROR: malloc failure\n\n" );	               
         if ( (lcase[anz->l].compName[i] = (char *)malloc( MAX_LINE_LENGTH * sizeof(char))) == NULL )
            printf("\n\n ERROR: malloc failed\n\n" );
@@ -1295,8 +1330,8 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
       /* nr of nodes per block can be read from the frd file, this is not documented in the original frd-spec. */
       nod_data=stoi( rec_str, 25, 36 );
       printf ("scip %d nodes\n",nod_data);
-      if(format_flag == 2) fseek(handle, nod_data*(sizeof(int)+3*sizeof(float)), SEEK_CUR);
-      else  fseek(handle, nod_data*(sizeof(int)+3*sizeof(double)), SEEK_CUR);
+      if(format_flag == 2) fseeko(handle, nod_data*(sizeof(int)+3*sizeof(float)), SEEK_CUR);
+      else  fseeko(handle, nod_data*(sizeof(int)+3*sizeof(double)), SEEK_CUR);
     }
     else if(flag == -3)
     {
@@ -1305,9 +1340,9 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
       printf ("scip %d elements\n",elem_data);
       for (i=0; i<elem_data; i++)
       {
-        fseek(handle, sizeof(int), SEEK_CUR);
+        fseeko(handle, sizeof(int), SEEK_CUR);
         length=fread((int *)&j,sizeof(int),1,handle);
-        fseek(handle, 2*sizeof(int), SEEK_CUR);
+        fseeko(handle, 2*sizeof(int), SEEK_CUR);
         if (j == 1)      ipuf = 8;   /* HEXA8  */
         else if (j == 2) ipuf = 6;   /* PE6   */
         else if (j == 3) ipuf = 4;   /* TET4   */
@@ -1320,7 +1355,7 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
         else if (j == 10) ipuf = 8; /* QUAD8  */
         else if (j == 11) ipuf = 2;  /* BEAM2   */
         else if (j == 12) ipuf = 3;  /* BEAM3   */
-        fseek(handle, ipuf*sizeof(int), SEEK_CUR);
+        fseeko(handle, ipuf*sizeof(int), SEEK_CUR);
       }
     }
     else
@@ -1349,12 +1384,12 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
 
   if ( e_nmax > (anz->nmax) )
   {
-    printf ("\nWARNING: element requestes a nodename higher than allocated\n\n");
+    printf ("\nWARNING: element requests a nodename higher than allocated\n\n");
     printf (" e_nmax=%d e_nmin=%d\n", e_nmax, e_nmin );
   }
   if ( e_nmin < 1 )
   {
-    printf ("\nWARNING: element requestes a nodename lower than allocated\n\n");
+    printf ("\nWARNING: element requests a nodename lower than allocated\n\n");
     printf (" e_nmax=%d e_nmin=%d\n", e_nmax, e_nmin );
   }
 
@@ -1370,14 +1405,15 @@ int readfrd( char *datin, Summen *anz, Nodes **nptr, Elements **eptr, Datasets *
 
 /* if a block was skipped during first read of frd-file read it now */
 /* return -1 if failure */
-int readfrdblock(int lc, Summen *anz,   Nodes     *node, Datasets *lcase )
+int readfrdblock(int lc, Summen *anz,   Nodes     *node,Datasets *lcase )
 {
-  register int i,j, n;
-  int  length, flag, format_flag, nodenr=0, ncomps;
+  int i,j, n;
+  int  length, flag, format_flag, nodenr=0, ncomps, nvals;
   int  nod_data=0, maxcomps=0;
   int  errFlag=0;
   char rec_str[MAX_LINE_LENGTH];
   static float *value=NULL;
+  static double *dvalue=NULL;
   FILE *handle;
 
   // printf("readfrdblock for file:%s\n", lcase[lc].filename);
@@ -1404,11 +1440,11 @@ int readfrdblock(int lc, Summen *anz,   Nodes     *node, Datasets *lcase )
     return(-1);
   }
 
-  if ( (lcase[lc].dat = (float **)malloc( (lcase[lc].ncomps) * sizeof(float *))) == NULL )
+  if ( (lcase[lc].dat = (CGXFLOAT **)malloc( (lcase[lc].ncomps) * sizeof(CGXFLOAT *))) == NULL )
     printf("\n\n ERROR: malloc failure\n\n" );
   for(i=0; i<(lcase[lc].ncomps); i++)
   {
-    if ( (lcase[lc].dat[i] = (float *)malloc( (anz->nmax+1) * sizeof(float))) == NULL )
+    if ( (lcase[lc].dat[i] = (CGXFLOAT *)malloc( (anz->nmax+1) * sizeof(CGXFLOAT))) == NULL )
       printf("\n\n ERROR: malloc failure\n\n" );	               
     for(j=0; j<=anz->nmax; j++) lcase[lc].dat[i][j]=0.;
   }
@@ -1426,7 +1462,7 @@ int readfrdblock(int lc, Summen *anz,   Nodes     *node, Datasets *lcase )
       flag = stoi(rec_str,1,3);
 
       /* bin mode, active after last column definition was read (-5 lines). Attention flag:-6 not permitted so far! */
-      if ( format_flag==2)
+      if ( format_flag>=2)
       {
         if (flag == -4)
         {
@@ -1439,19 +1475,45 @@ int readfrdblock(int lc, Summen *anz,   Nodes     *node, Datasets *lcase )
         /* skip the meta-data */
         for(i=0; i<ncomps; i++) length = frecord( handle, rec_str);
 
-        if ( (value = (float *)realloc((float *)value, (lcase[lc].ncomps) * sizeof(float))) == NULL )
-          printf("\n\n ERROR: realloc failed, value\n\n") ;
-        for(n=0; n<nod_data; n++)
+        if ( format_flag==2)
         {
-          length=fread((int *)&nodenr,sizeof(int),1,handle);
-          length=fread((float *)value,sizeof(float),lcase[lc].ncomps,handle);
-	  // printf("N:%d ",nodenr);
-          for(i=0; i<lcase[lc].ncomps; i++)
+          if ( (value = (float *)realloc((float *)value, (lcase[lc].ncomps) * sizeof(float))) == NULL )
+            printf("\n\n ERROR: realloc failed, value\n\n") ;
+          nvals=0;
+          for (i=0; i<lcase[lc].ncomps; i++) if(lcase[lc].iexist[i]!=1) nvals++;
+          for(n=0; n<nod_data; n++)
           {
-	    // printf(" %f",value[i]); 
-            lcase[lc].dat[i][nodenr]= value[i];
+            length=fread((int *)&nodenr,sizeof(int),1,handle);
+            length=fread((float *)value,sizeof(float),nvals,handle);
+  	  // printf("N:%d ",nodenr);
+            for(i=0; i<lcase[lc].ncomps; i++)
+            {
+  	    // printf(" %f",value[i]); 
+              if(lcase[lc].iexist[i]!=1) lcase[lc].dat[i][nodenr]= value[i];
+	      else lcase[lc].dat[i][nodenr]=0.;
+            }
+  	  // printf("\n");
+          }
+        }
+        if ( format_flag==3)
+        {
+          if ( (dvalue = (double *)realloc((double *)dvalue, (lcase[lc].ncomps) * sizeof(double))) == NULL )
+            printf("\n\n ERROR: realloc failed, dvalue\n\n") ;
+          nvals=0;
+          for (i=0; i<lcase[lc].ncomps; i++) if(lcase[lc].iexist[i]!=1) nvals++;
+          for(n=0; n<nod_data; n++)
+          {
+            length=fread((int *)&nodenr,sizeof(int),1,handle);
+            length=fread((double *)dvalue,sizeof(double),nvals,handle);
+  	  // printf("N:%d ",nodenr);
+            for(i=0; i<lcase[lc].ncomps; i++)
+            {
+  	    // printf(" %f",dvalue[i]); 
+              if(lcase[lc].iexist[i]!=1) lcase[lc].dat[i][nodenr]= dvalue[i];
+	      else lcase[lc].dat[i][nodenr]=0.;
+            }
+  	  // printf("\n");
           }
-	  // printf("\n");
         }
         break;        
       }
@@ -1540,7 +1602,7 @@ char *getRecord(FILE *handle, int n, int x0 )
       m=(n2+n1)/2;
 
       
-      if( fseek( handle, offset, SEEK_CUR )!=0) printf("error in fseek\n");
+      if( fseeko( handle, offset, SEEK_CUR )!=0) printf("error in fseeko\n");
                           
       if(x0>= x ) n1=m;              
       if(x0 < x ) n2=m;              
@@ -1564,7 +1626,8 @@ int readOneNode( int lc, Summen *anz, Datasets *lcase, int nodenr, double **vptr
   int  errFlag=0, readFlag=0, bailout=0;
   char rec_str[MAX_LINE_LENGTH];
   FILE *handle;
-  float *value=NULL;
+  static float *value=NULL;
+  static double *dvalue=NULL;
   double *dat;
   int  nod_data=0, nvals=0;
 
@@ -1577,7 +1640,7 @@ int readOneNode( int lc, Summen *anz, Datasets *lcase, int nodenr, double **vptr
   if( fsetpos( handle, (fpos_t *)lcase[lc].fileptr)!=0) { printf("error in fsetpos"); return(-1); }
 
   /* the header-lines must be skipped in case byte_offset==0 and format == 2(bin) */
-  if((lcase[lc].format_flag==2)&&(!*byte_offset))
+  if((lcase[lc].format_flag>=2)&&(!*byte_offset))
   {
     do
     {
@@ -1597,7 +1660,7 @@ int readOneNode( int lc, Summen *anz, Datasets *lcase, int nodenr, double **vptr
       break;        
     }while(1);
   }
-  else{ if( fseek( handle, *byte_offset, SEEK_CUR )!=0) printf("error in fseek\n"); }
+  else{ if( fseeko( handle, *byte_offset, SEEK_CUR )!=0) printf("error in fseeko\n"); }
 
   offset=*byte_offset;
 
@@ -1610,23 +1673,51 @@ int readOneNode( int lc, Summen *anz, Datasets *lcase, int nodenr, double **vptr
   {
     if ( (value = (float *)realloc((float *)value, (lcase[lc].ncomps) * sizeof(float))) == NULL )
       printf("\n\n ERROR: realloc failed, value\n\n") ;
+    nvals=n=0;
+    for (i=0; i<lcase[lc].ncomps; i++) if(lcase[lc].iexist[i]!=1) nvals++;
     do
     {
       length=fread((int *)&inodenr,sizeof(int),1,handle)*sizeof(int);
-      length+=fread((float *)value,sizeof(float),lcase[lc].ncomps,handle)*sizeof(float);
+      length+=fread((float *)value,sizeof(float),nvals,handle)*sizeof(float);
       //printf("N:%d ",inodenr);
-      //for(i=0; i<lcase[lc].ncomps; i++) printf(" %f",value[i]);
+      //for(i=0; i<nvals; i++) printf(" %f",value[i]);
       //printf("\n");
       if(inodenr==nodenr) break;
       else offset+=length;
-    }while(1);
-    for(i=0; i<lcase[lc].ncomps; i++)
+    }while(++n<lcase[lc].numnod);
+    for(i=0; i<nvals; i++)
     {
 	//printf(" %f",value[i]);
       dat[i]= value[i];
     }
 
-    *byte_offset=offset;
+    //*byte_offset=offset;
+    //printf("offset:%d\n", offset);
+    return(0);
+  }
+  if(lcase[lc].format_flag==3)
+  {
+    if ( (dvalue = (double *)realloc((double *)dvalue, (lcase[lc].ncomps) * sizeof(double))) == NULL )
+      printf("\n\n ERROR: realloc failed, dvalue\n\n") ;
+    nvals=n=0;
+    for (i=0; i<lcase[lc].ncomps; i++) if(lcase[lc].iexist[i]!=1) nvals++;
+    do
+    {
+      length=fread((int *)&inodenr,sizeof(int),1,handle)*sizeof(int);
+      length+=fread((double *)dvalue,sizeof(double),nvals,handle)*sizeof(double);
+      //printf("N:%d ",inodenr);
+      //for(i=0; i<nvals; i++) printf(" %f",dvalue[i]);
+      //printf("\n");
+      if(inodenr==nodenr) break;
+      else offset+=length;
+    }while(++n<lcase[lc].numnod);
+    for(i=0; i<nvals; i++)
+    {
+	//printf(" %f",dvalue[i]);
+      dat[i]= dvalue[i];
+    }
+
+    //*byte_offset=offset;
     //printf("offset:%d\n", offset);
     return(0);
   }
