@@ -23,7 +23,7 @@
 
 #include <extUtil.h>
 
-int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, char **dat )
+int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets *lcase, char **dat, int controlFlag )
 {
   FILE *handle1=NULL, *handle2=NULL;
   int  i,j, lc, n, lendatout;
@@ -125,6 +125,7 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
         if((i==0) || ((i>0)&&(elem[i-1].type!=elem[i].type)) || ((i>0)&&(elem[i-1].attr!=elem[i].attr)))
         {
           if (elem[i].attr==8) fprintf (handle1, "*ELEMENT, TYPE=C3D10M, ELSET=E%s\n", anz->model);
+          else if (elem[i].attr==9) fprintf (handle1, "*ELEMENT, TYPE=C3D10T, ELSET=E%s\n", anz->model);
           else fprintf (handle1, "*ELEMENT, TYPE=C3D10, ELSET=E%s\n", anz->model);
         }
         fprintf (handle1, "%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d,%6d \n",
@@ -194,7 +195,6 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
       }
       else if (elem[i].type == 11)
       {
-	printf("e:%d attr:%d\n", elem[i].nr,elem[i].attr);
         if((i==0) || ((i>0)&&(elem[i-1].type!=elem[i].type)) || ((i>0)&&(elem[i-1].attr!=elem[i].attr)))
         {
           if (elem[i].attr==3)  fprintf (handle1, "*ELEMENT, TYPE=DASHPOTA, ELSET=E%s\n", anz->model);
@@ -223,6 +223,7 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
 	    }
             continue;
 	  }
+          else if (elem[i].attr==1) fprintf (handle1, "*ELEMENT, TYPE=B31R, ELSET=E%s\n", anz->model);
           else  fprintf (handle1, "*ELEMENT, TYPE=B31, ELSET=E%s\n", anz->model);
         }
         fprintf (handle1, "%6d, %6d, %6d\n",
@@ -233,7 +234,8 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
         if((i==0) || ((i>0)&&(elem[i-1].type!=elem[i].type)) || ((i>0)&&(elem[i-1].attr!=elem[i].attr)))
         {
           if (elem[i].attr==7) fprintf (handle1, "*ELEMENT, TYPE=D, ELSET=E%s\n", anz->model);
-	  else fprintf (handle1, "*ELEMENT, TYPE=B32R, ELSET=E%s\n", anz->model);
+	  else if (elem[i].attr==1)  fprintf (handle1, "*ELEMENT, TYPE=B32R, ELSET=E%s\n", anz->model);
+	  else fprintf (handle1, "*ELEMENT, TYPE=B32, ELSET=E%s\n", anz->model);
         }
         fprintf (handle1, "%6d, %6d, %6d, %6d\n",
           elem[i].nr, elem[i].nod[0], elem[i].nod[2], elem[i].nod[1] );
@@ -266,6 +268,16 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
     /* therefore a loop over all Datasets is performed twice */
     if((anz->l>0)&&(dat!=0))
     {
+     if((compare( dat[0], "crp", 3)== 3)||(compare( dat[0], "sta", 3)== 3))
+     {
+       fprintf (handle2, "** The files ccx.clo, ccx.dlo have to store the loading at the reference speed\n");
+       fprintf (handle2, "** If CENRTIF should be regarded, incl the following line in ccx.dlo\n");
+       fprintf (handle2, "** Eall, CENTRIF, %f,  0., 0., 0.,  1., 0., 0\n", frc2*(atof(dat[2])/60.*2*PI)*(atof(dat[2])/60.*2*PI));
+       printf (" The files ccx.clo, ccx.dlo have to store the loading at the reference speed\n");
+       printf (" If CENRTIF should be regarded, incl the following line in ccx.dlo\n");
+       printf (" Eall, CENTRIF, %f,  0., 0., 0.,  1., 0., 0\n", frc2*(atof(dat[2])/60.*2*PI)*(atof(dat[2])/60.*2*PI));
+     }
+
      /* first loop for the *AMPLITUDEs */
      j=0;
      for (lc=0; lc<anz->l; lc++)
@@ -325,7 +337,8 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
           fprintf (handle2, "*STEP\n");
           /* fprintf (handle2, "*STATIC, SOLVER=SPOOLES\n"); */
           fprintf (handle2, "*STATIC\n");
-          fprintf (handle2, "%lf, %lf,,,\n", time2, time2);
+	  if((j==1)&&(time2<0.0001)) fprintf (handle2, "0.0002, 0.001,,,\n");
+          else fprintf (handle2, "%lf, %lf,,,\n", time2, time2);
           fprintf (handle2, "*TEMPERATURE\n");
         }
         if(compare( dat[0], "crp", 3)== 3)
@@ -341,13 +354,13 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
           tmin  = time2/10.;
           tmax  = time2;
           fprintf (handle2, "** ---  LC%d ---\n", j);
-          fprintf (handle2, "*STEP, NLGEOM, AMPLITUDE=RAMP\n");
-          fprintf (handle2, "*VISCO, CETOL=5.e-4\n");
+          fprintf (handle2, "*STEP, NLGEOM\n");
+          fprintf (handle2, "*VISCO\n");
           fprintf (handle2, "%lf, %lf,,,\n", tmin, tmax);
           fprintf (handle2, "*CLOAD, AMPLITUDE=FORCE%d\n", j);
           fprintf (handle2, "*include, input=ccx.clo\n");
-          fprintf (handle2, "*DLOAD\n");
-          fprintf (handle2, "Eall, CENTRIF, %f,  0., 0., 0.,  1., 0., 0\n", frc2*(atof(dat[2])/60.*2*PI)*(atof(dat[2])/60.*2*PI));
+          //fprintf (handle2, "*DLOAD\n");
+          //fprintf (handle2, "Eall, CENTRIF, %f,  0., 0., 0.,  1., 0., 0\n", frc2*(atof(dat[2])/60.*2*PI)*(atof(dat[2])/60.*2*PI));
           fprintf (handle2, "*DLOAD, AMPLITUDE=FORCE%d\n", j);
           fprintf (handle2, "*include, input=ccx.dlo\n");
           fprintf (handle2, "*TEMPERATURE\n");
@@ -365,8 +378,8 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
           fprintf (handle2, "%lf, %lf,,,\n", time2, time2);
           fprintf (handle2, "*CLOAD, AMPLITUDE=FORCE%d\n", j);
           fprintf (handle2, "*include, input=ccx.clo\n");
-          fprintf (handle2, "*DLOAD\n");
-          fprintf (handle2, "Eall, CENTRIF, %f,  0., 0., 0.,  1., 0., 0\n", frc2*(atof(dat[2])/60.*2*PI)*(atof(dat[2])/60.*2*PI));
+          //fprintf (handle2, "*DLOAD\n");
+          //fprintf (handle2, "Eall, CENTRIF, %f,  0., 0., 0.,  1., 0., 0\n", frc2*(atof(dat[2])/60.*2*PI)*(atof(dat[2])/60.*2*PI));
           fprintf (handle2, "*DLOAD, AMPLITUDE=FORCE%d\n", j);
           fprintf (handle2, "*include, input=ccx.dlo\n");
           fprintf (handle2, "*TEMPERATURE\n");
@@ -374,18 +387,19 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
   
         for (i=0; i<anz->n; i++)
         {
-          fprintf (handle2, "%8d, %12.5f\n", node[i].nr,
-          lcase[lc].dat[0][node[i].nr]);
+          fprintf (handle2, "%8d, %.7e\n", node[i].nr, lcase[lc].dat[0][node[i].nr]);
         }
         
         if( (compare( dat[0], "tmf", 3)== 3) || (compare( dat[0], "sta", 3)== 3) )
         {
           fprintf (handle2, "*NODE PRINT, FREQUENCY=0 \n");
           fprintf (handle2, "*EL PRINT, FREQUENCY=0 \n");
-          fprintf (handle2, "*NODE FILE\n");
+          fprintf (handle2, "*NODE FILE, FREQUENCY=99\n");
           fprintf (handle2, "U, NT \n");
-          fprintf (handle2, "*EL FILE,POSITION=AVERAGED AT NODES\n");
+          fprintf (handle2, "*EL FILE, FREQUENCY=99,POSITION=AVERAGED AT NODES\n");
           fprintf (handle2, "S,E,ME \n");
+          fprintf (handle2, "*CONTACT FILE, FREQUENCY=99\n");
+          fprintf (handle2, "CDISP,CSTRESS\n");
           fprintf (handle2, "*END STEP\n");
         }
         if(compare( dat[0], "crp", 3)== 3)
@@ -396,6 +410,8 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
           fprintf (handle2, "U, NT \n");
           fprintf (handle2, "*EL FILE, FREQUENCY=%d,POSITION=AVERAGED AT NODES\n", atoi(dat[3]));
           fprintf (handle2, "S, CE \n");
+          fprintf (handle2, "*CONTACT FILE\n");
+          fprintf (handle2, "CDISP,CSTRESS\n");
           fprintf (handle2, "*END STEP\n");
         }
       }
@@ -403,16 +419,30 @@ int write2aba( char *datout, Summen *anz, Nodes *node, Elements *elem, Datasets
       {
         if((compare( dat[0], "ds", 2)== 2)&&(atoi(&dat[0][2])==lc+1))
         {
-          for (n=0; n<lcase[lc].ncomps; n++)
+          if((dat[1]!=(char *)NULL)&&(dat[1][0]=='e'))
           {
-	    if(lcase[lc].iexist[n]!=0) continue;
+            n=atoi(&dat[1][1]);
+            n--;
+	    if(n>=lcase[lc].ncomps) continue;
             fprintf (handle2, "** %s %s\n", lcase[lc].name,lcase[lc].compName[n] );
             for (i=0; i<anz->n; i++)
             {
-	      if(lcase[lc].ncomps==1) fprintf (handle2, "%8d, %f\n", node[i].nr, lcase[lc].dat[n][node[i].nr]);
-              else fprintf (handle2, "%8d,%d,%f\n", node[i].nr, n+1, lcase[lc].dat[n][node[i].nr]);
+              fprintf (handle2, "%8d, %.7e\n", node[i].nr, lcase[lc].dat[n][node[i].nr]);
             }
           }
+          else
+	  {
+           for (n=0; n<lcase[lc].ncomps; n++)
+           {
+	    if((controlFlag!=2)&&(lcase[lc].iexist[n]!=0)) continue;
+            fprintf (handle2, "** %s %s\n", lcase[lc].name,lcase[lc].compName[n] );
+            for (i=0; i<anz->n; i++)
+            {
+	      if(lcase[lc].ncomps==1) fprintf (handle2, "%8d, %.7e\n", node[i].nr, lcase[lc].dat[n][node[i].nr]);
+              else fprintf (handle2, "%8d,%d,%.7e\n", node[i].nr, n+1, lcase[lc].dat[n][node[i].nr]);
+            }
+           }
+          }
         }
       }
     }
