@@ -32,13 +32,13 @@ template < class T > class ptrListItem
         virtual ~ptrListItem();
         
         ptrListItem();
-        ptrListItem ( const ptrListItem<T>& item );
+        ptrListItem ( ptrListItem<T>& item );
         
         ptrListItem ( T* ptr, ptrListItem<T>* chain, bool delObjPtdTo, bool insert = false );
 
         // Operators
 
-        ptrListItem<T>& operator= ( const ptrListItem<T>& item );
+        ptrListItem<T>& operator= ( ptrListItem<T>& item );
 
         // Linked list functions.
         ptrListItem<T>* next();
@@ -137,7 +137,7 @@ template < class T > ptrListItem<T>::ptrListItem()
     nextLnk = 0;
 }
 
-template < class T > ptrListItem<T>::ptrListItem ( const ptrListItem<T>& item )
+template < class T > ptrListItem<T>::ptrListItem ( ptrListItem<T>& item )
 {
     // Copy constructor.
     // -----------------
@@ -200,7 +200,7 @@ template < class T > ptrListItem<T>::ptrListItem ( T* ptr, ptrListItem<T>* chain
         prevLnk = 0;
 }
 
-template < class T > ptrListItem<T>& ptrListItem<T>::operator= ( const ptrListItem<T>& item )
+template < class T > ptrListItem<T>& ptrListItem<T>::operator= ( ptrListItem<T>& item )
 {
     // Assignment Operator.
     // --------------------
